
#include "stdafx.h"

#include "CDListFile.h"
#include "Discography.h"

/////////////////////////////////////////////////////////////////////////

CCDListFileLine::CCDListFileLine(void)
{
	m_lineBuffer = NULL;
	m_artistBuffer = NULL;
	m_artistShortBuffer = NULL;
	m_titleBuffer = NULL;

	m_number = 0;
	m_artist = NULL;
	m_title = NULL;
}

CCDListFileLine::~CCDListFileLine(void)
{
	SetLineBuffer(NULL);
	setArtistBuffer(NULL);
	setArtistShortBuffer(NULL);
	setTitleBuffer(NULL);
}

void CCDListFileLine::SetLineBuffer(const char *buffer)
{
	if (NULL != m_lineBuffer)
	{
		delete [] m_lineBuffer;
		m_lineBuffer = NULL;
	}

	if (NULL != buffer)
	{
		m_lineBuffer = new char[strlen(buffer) + 1];
		strcpy(m_lineBuffer, buffer);
	}
}

void CCDListFileLine::setArtistBuffer(const char *buffer)
{
	if (NULL != m_artistBuffer)
	{
		delete [] m_artistBuffer;
		m_artistBuffer = NULL;
	}

	if (NULL != buffer)
	{
		m_artistBuffer = new char[strlen(buffer) + 1];
		strcpy(m_artistBuffer, buffer);
	}
}

void CCDListFileLine::setArtistShortBuffer(const char *buffer)
{
	if (NULL != m_artistShortBuffer)
	{
		delete [] m_artistShortBuffer;
		m_artistShortBuffer = NULL;
	}

	if (NULL != buffer)
	{
		m_artistShortBuffer = new char[strlen(buffer) + 1];
		strcpy(m_artistShortBuffer, buffer);
	}
}

void CCDListFileLine::setTitleBuffer(const char *buffer)
{
	if (NULL != m_titleBuffer)
	{
		delete [] m_titleBuffer;
		m_titleBuffer = NULL;
	}

	if (NULL != buffer)
	{
		m_titleBuffer = new char[strlen(buffer) + 1];
		strcpy(m_titleBuffer, buffer);
	}
}

bool CCDListFileLine::ParseBuffer(unsigned int previousNumber, const char *previousArtist)
{
	bool bResult = true;

	_ASSERTE(NULL != m_lineBuffer);

	const char *bufPtr = m_lineBuffer;
	const char *delimiters = "~";
	size_t tokenBufferSize = strlen(m_lineBuffer) + 1;
	char *tokenBuffer = new char[tokenBufferSize];

	// Disc Number
	if (bResult)
	{
		if (!TokenizeEx(&bufPtr, delimiters, tokenBuffer, tokenBufferSize, false))
		{
			printf("ERROR: Invalid line in input file (no disc number column):\n%s\n", m_lineBuffer);
			bResult = false;
		}
		else if (0 == strlen(tokenBuffer))
		{
			// Empty number column, use previousNumber+1;
			m_number = previousNumber + 1;
		}
		else
		{
			m_number = atoi(tokenBuffer);
		}
	}

	// Artist
	if (bResult)
	{
		if (!TokenizeEx(&bufPtr, delimiters, tokenBuffer, tokenBufferSize, false))
		{
			printf("ERROR: Invalid line in input file (no artist column):\n%s\n", m_lineBuffer);
			bResult = false;
		}
		else if (0 == strlen(tokenBuffer))
		{
			// Empty artist column, use previousArtist.
			m_artist = previousArtist;
		}
		else
		{
			setArtistBuffer(tokenBuffer);
			m_artist = m_artistBuffer;
		}
	}

	// Short Artist Name
	if (bResult)
	{
		if (!TokenizeEx(&bufPtr, delimiters, tokenBuffer, tokenBufferSize, false))
		{
			printf("ERROR: invalid line in input file (no short artist column):\n%s\n", m_lineBuffer);
			bResult = false;
		}
		else if (0 != strlen(tokenBuffer))
		{
			setArtistShortBuffer(tokenBuffer);
		}
	}

	// Title
	if (bResult)
	{
		if (!TokenizeEx(&bufPtr, delimiters, tokenBuffer, tokenBufferSize, false))
		{
			printf("ERROR: Invalid line in input file (no title column):\n%s\n", m_lineBuffer);
			bResult = false;
		}
		else if (0 == strlen(tokenBuffer))
		{
			printf("ERROR: Invalid line in input file (empty title column):\n%s\n", m_lineBuffer);
			bResult = false;
		}
		else
		{
			setTitleBuffer(tokenBuffer);
			m_title = m_titleBuffer;
		}
	}

	delete [] tokenBuffer;

	// Free line buffer.
	if (bResult)
	{
		SetLineBuffer(NULL);
	}

	return(bResult);
}

unsigned int CCDListFileLine::GetNumber(void)
{
	return(m_number);
}

const char *CCDListFileLine::GetArtist(void)
{
	return(m_artist);
}

const char *CCDListFileLine::GetArtistShort(void)
{
	return(m_artistShortBuffer);
}

const char *CCDListFileLine::GetTitle(void)
{
	return(m_title);
}

/////////////////////////////////////////////////////////////////////////

CCDListFile::CCDListFile(void)
{
}

CCDListFile::~CCDListFile(void)
{
	while (!m_lines.empty())
	{
		delete m_lines.back();
		m_lines.pop_back();
	}
}

bool CCDListFile::Read(FILE *pfInputList)
{
	bool bResult = true;

	// At some point, replace this fixed-size buffer and fgets() with something more intelligent.
	char lineBuffer[BUFFERSIZE];

	char *trimmedLine;
	CCDListFileLine *pLine;

	while (NULL != fgets(lineBuffer, sizeof(lineBuffer), pfInputList))
	{
		trimmedLine = TrimWhitespace(lineBuffer);

		// Ignore empty lines and those starting with "//"
		if ( (0 < strlen(trimmedLine)) && (('/' != trimmedLine[0]) || ('/' != trimmedLine[1])) )
		{
			pLine = new CCDListFileLine();
			pLine->SetLineBuffer(trimmedLine);

			m_lines.push_back(pLine);
		}
	}

	if (bResult && ferror(pfInputList))
	{
		printf("ERROR: Cannot read input file\n");
		bResult = false;
	}

	return(bResult);
}

bool CCDListFile::Parse(void)
{
	bool bResult = true;

	unsigned int previousNumber = 0;				// CD changers seem to start at 1.
	const char *previousArtist = "Unknown Artist";	// A good default that also saves error checks.

	list<CCDListFileLine *>::iterator iter;

	for(iter = m_lines.begin(); m_lines.end() != iter; iter++)
	{
		if ((*iter)->ParseBuffer(previousNumber, previousArtist))
		{
			previousNumber = (*iter)->GetNumber();
			previousArtist = (*iter)->GetArtist();
		}
		else
		{
			bResult = false;
			break;
		}
	}

	return(bResult);
}

bool CCDListFile::MakeArtistList(CArtistList *pArtistList)
{
	bool bResult = true;

	list<CCDListFileLine *>::const_iterator iter;

	CArtist *pArtist;

	for(iter = m_lines.begin(); iter != m_lines.end(); iter++)
	{
		pArtist = pArtistList->FindElseAdd((*iter)->GetArtist());

		_ASSERTE(NULL != pArtist);

		if (NULL != (*iter)->GetArtistShort())
		{
			pArtist->SetShortName((*iter)->GetArtistShort());
		}

		pArtist->GetDiscList()->AddTail((*iter)->GetNumber(),
										(*iter)->GetTitle());
	}

	pArtistList->Sort();

	return(bResult);
}

/////////////////////////////////////////////////////////////////////////
