
#include "stdafx.h"

#include "CSFSection.h"

#include "UnsignedBaseTenNumber.h"

/////////////////////////////////////////////////////////////////////////

CCSFButtonMemo::CCSFButtonMemo(const char *device, const char *panel, const char *name, unsigned int number)
{
	if (NULL == device)
	{
		m_device = new char[1];
		m_device[0] = '\0';
	}
	else
	{
		m_device = new char[strlen(device) + 1];
		strcpy(m_device, device);
	}

	if (NULL == panel)
	{
		m_panel = new char[1];
		m_panel[0] = '\0';
	}
	else
	{
		m_panel = new char[strlen(panel) + 1];
		strcpy(m_panel, panel);
	}

	if (NULL == name)
	{
		m_name = new char[1];
		m_name[0] = '\0';
	}
	else
	{
		m_name = new char[strlen(name) + 1];
		strcpy(m_name, name);
	}

	m_number = number;

	m_aliasString = NULL;

	generateAliasString();
}

CCSFButtonMemo::~CCSFButtonMemo(void)
{
	if (NULL != m_device)
	{
		delete [] m_device;
		m_device = NULL;
	}

	if (NULL != m_panel)
	{
		delete [] m_panel;
		m_panel = NULL;
	}

	if (NULL != m_name)
	{
		delete [] m_name;
		m_name = NULL;
	}

	if (NULL != m_aliasString)
	{
		delete [] m_aliasString;
		m_aliasString = NULL;
	}
}

const char *CCSFButtonMemo::GetDevice(void) const
{
	return(m_device);
}

const char *CCSFButtonMemo::GetPanel(void) const
{
	return(m_panel);
}

const char *CCSFButtonMemo::GetName(void) const
{
	return(m_name);
}

unsigned int CCSFButtonMemo::GetNumber(void) const
{
	return(m_number);
}

const char *CCSFButtonMemo::GetAliasString(void) const
{
	return(m_aliasString);
}

void CCSFButtonMemo::generateAliasString(void)
{
	CUnsignedBaseTenNumber unsignedBaseTenNumber(m_number);

	const char *part1 = "AliasDeviceButton(\"";
	const char *part2 = "\",\"";
	const char *part3 = "\",\"";
	const char *part4 =	"\",";
	const char *part5 = "); /* Reference to Button ";
	const char *part6 = " */";

	size_t bufferSize = 1;

	bufferSize += strlen(part1);
	bufferSize += strlen(m_device);
	bufferSize += strlen(part2);
	bufferSize += strlen(m_panel);
	bufferSize += strlen(part3);
	bufferSize += strlen(m_name);
	bufferSize += strlen(part4);
	bufferSize += unsignedBaseTenNumber.GetNumberOfDigits();
	bufferSize += strlen(part5);
	bufferSize += unsignedBaseTenNumber.GetNumberOfDigits();
	bufferSize += strlen(part6);

	if (NULL != m_aliasString)
	{
		delete [] m_aliasString;
		m_aliasString = NULL;
	}

	m_aliasString = new char[bufferSize];

	stringFormat(m_aliasString, bufferSize, "%s%s%s%s%s%s%s%s%s%s%s",
					part1, m_device, part2, m_panel, part3, m_name, part4,
					unsignedBaseTenNumber.GetAsciiRepresentation(),
					part5,
					unsignedBaseTenNumber.GetAsciiRepresentation(),
					part6);
}

/////////////////////////////////////////////////////////////////////////

CCSFEnvironment::CCSFEnvironment(void)
{
	m_nextButtonNumber = 1;

	m_pBMDisc = NULL;
	m_pBMEnter = NULL;

	for (unsigned int i=0; i < 10; i++)
	{
		m_pBMNumbers[i] = NULL;
	}
}

CCSFEnvironment::~CCSFEnvironment(void)
{
	SetBMDisc(NULL);
	SetBMEnter(NULL);

	for (unsigned int i=0; i < 10; i++)
	{
		SetBMNumber(i, NULL);
	}
}

unsigned int CCSFEnvironment::GetAndIncrementNextButtonNumber(void)
{
	return(m_nextButtonNumber++);
}

void CCSFEnvironment::UpdateNextButtonNumber(unsigned int foundButtonNumber)
{
	if (m_nextButtonNumber <= foundButtonNumber)
	{
		m_nextButtonNumber = foundButtonNumber + 1;
	}
}

CCSFButtonMemo *CCSFEnvironment::GetBMDisc(void) const
{
	return(m_pBMDisc);
}

CCSFButtonMemo *CCSFEnvironment::GetBMEnter(void) const
{
	return(m_pBMEnter);
}

CCSFButtonMemo *CCSFEnvironment::GetBMNumber(unsigned int number) const
{
	CCSFButtonMemo *result = NULL;

	if ((0 <= number) && (number <= 9)) // First check not really needed for unsigned. :-)
	{
		result = m_pBMNumbers[number];
	}

	return(result);
}

void CCSFEnvironment::SetBMDisc(CCSFButtonMemo *pBM)
{
	if (NULL != m_pBMDisc)
	{
		delete m_pBMDisc;
		m_pBMDisc = NULL;
	}

	m_pBMDisc = pBM;
}

void CCSFEnvironment::SetBMEnter(CCSFButtonMemo *pBM)
{
	if (NULL != m_pBMEnter)
	{
		delete m_pBMEnter;
		m_pBMEnter = NULL;
	}

	m_pBMEnter = pBM;
}

void CCSFEnvironment::SetBMNumber(unsigned int number, CCSFButtonMemo *pBM)
{
	if ((0 <= number) && (number <= 9)) // First check not really needed for unsigned. :-)
	{
		if (NULL != m_pBMNumbers[number])
		{
			delete m_pBMNumbers[number];
			m_pBMNumbers[number] = NULL;
		}

		m_pBMNumbers[number] = pBM;
	}
}


/////////////////////////////////////////////////////////////////////////

CCSFSection::CCSFSection(CCSFSection *pSectionParent)
{
	m_pSectionParent = pSectionParent;

	if (NULL == m_pSectionParent)
	{
		m_pEnvironment = new CCSFEnvironment();
	}
	else
	{
		m_pSectionParent->m_bHasSubBody = true;
		m_pSectionParent->m_subSections.push_back(this);
		m_pEnvironment = m_pSectionParent->GetEnvironment();
	}

	m_title = NULL;
	m_bHasSubBody = false;
}

CCSFSection::~CCSFSection(void)
{
	SetTitle(NULL);

	while (! m_subSections.empty() )
	{
		delete m_subSections.back();
		m_subSections.pop_back();
	}

	if (NULL == m_pSectionParent)
	{
		// We are the root, we created the nextButtonNumber and must delete it.
		delete m_pEnvironment;
	}
}

CCSFEnvironment *CCSFSection::GetEnvironment(void)
{
	return(m_pEnvironment);
}

CCSFSection *CCSFSection::GetParent(void)
{
	return(m_pSectionParent);
}

const CCSFSection *CCSFSection::GetParent(void) const
{
	return(m_pSectionParent);
}

const char *CCSFSection::GetTitle(void)
{
	return(m_title);
}

void CCSFSection::SetTitle(const char *title)
{
	if (NULL != m_title)
	{
		delete [] m_title;
		m_title = NULL;
	}

	if (NULL != title)
	{
		m_title = new char[strlen(title) + 1];
		strcpy(m_title, title);

		updateNextButtonNumber();
	}
}

void CCSFSection::SetTitleFormat(const char *format, ...)
{
	if (NULL == format)
	{
		SetTitle(NULL);
	}
	else
	{
		char buffer[BUFFERSIZE];

		va_list args;
		va_start(args, format);
		vStringFormat(buffer, sizeof(buffer), format, args);
		va_end(args);

		SetTitle(buffer);
	}
}

CCSFSection::subIterator CCSFSection::subBegin(void)
{
	return(m_subSections.begin());
}

CCSFSection::subIterator CCSFSection::subEnd(void)
{
	return(m_subSections.end());
}

CCSFSection::const_subIterator CCSFSection::subBegin(void) const
{
	return(m_subSections.begin());
}

CCSFSection::const_subIterator CCSFSection::subEnd(void) const
{
	return(m_subSections.end());
}

bool CCSFSection::ReadNextSection(CSimpleTextFile::const_lineIterator &lineIter,
								  const CSimpleTextFile::const_lineIterator &lineIterEnd)
{
	bool bResult = true;

	const char *lineBuffer;


	// Skip empty lines. (Whitespace has already been trimmed.)
	while(true)
	{
		if (lineIterEnd == lineIter)
		{
			printf("ERROR: End of input CSF file reached unexpectedly\n");
			bResult = false;
			break;
		}
		else
		{
			lineBuffer = (*lineIter)->GetLineBuffer();

			if (0 != strlen(lineBuffer))
			{
				break;
			}
		}
	}

	if (bResult)
	{
		SetTitle(lineBuffer);

		lineIter++;

		if ((lineIterEnd != lineIter) && ('{' == (*lineIter)->GetLineBuffer()[0]))
		{
			lineIter++;

			m_bHasSubBody = true; // So we output the braces even if the subbody is empty.

			CCSFSection *pNewSubSection;

			while(true)
			{
				if (lineIterEnd == lineIter)
				{
					printf("ERROR: End of input CSF file reached unexpectedly\n");
					bResult = false;
					break;
				}
				else
				{
					lineBuffer = (*lineIter)->GetLineBuffer();

					if (0 == strlen(lineBuffer)) // avoids missing the closing "}" if a blank line is right before it.
					{
						lineIter++;
					}
					else
					{
						if ('{' == lineBuffer[0])
						{
							printf("ERROR: Unexpected '{' in input CSF file\n");
							bResult = false;
							break;
						}
						else if ('}' == lineBuffer[0])
						{
							// Finished reading the subsection.
							lineIter++;
							break;
						}
						else
						{
							pNewSubSection = AddSubSection();

							if (! pNewSubSection->ReadNextSection(lineIter, lineIterEnd) )
							{
								bResult = false;
							}
						}
					}
				}
			}
		}
	}

	return(bResult);
}

bool CCSFSection::Write(FILE *pfOutputCSF, int indentLevel/* = 0*/) const
{
	if ((NULL != m_title) && ('\0' != m_title[0]))
	{
		indentLine(pfOutputCSF, indentLevel);

		fprintf(pfOutputCSF, "%s\n", (const char *)m_title);

		if (m_bHasSubBody)
		{
			indentLine(pfOutputCSF, indentLevel);
			fprintf(pfOutputCSF, "{\n");

			for (const_subIterator iter = subBegin(); subEnd() != iter; iter++)
			{
				(*iter)->Write(pfOutputCSF, indentLevel + 1);
			}

			indentLine(pfOutputCSF, indentLevel);
			fprintf(pfOutputCSF, "}\n");
		}
	}

	return(true);
}

void CCSFSection::indentLine(FILE *pfOutputCSF, int indentLevel) const
{
	for (unsigned int i=0; i < indentLevel; i++)
	{
		fprintf(pfOutputCSF, "\t");
//		fprintf(pfOutputCSF, "  ");
	}
}

// Do not delete the subsection once it has been added, the class will on destruction.
// Do not add the same subsection pointer to more than one CCSFSection.
CCSFSection *CCSFSection::AddSubSection(void)
{
	CCSFSection *result = new CCSFSection(this);

	return(result);
}

CCSFSection *CCSFSection::AddTitleOnlySubSection(const char *subTitle)
{
	CCSFSection *result = AddSubSection();

	result->SetTitle(subTitle);

	return(result);
}

CCSFSection *CCSFSection::AddTitleOnlySubSectionFormat(const char *format, ...)
{
	CCSFSection *result = AddSubSection();

	if (NULL != format)
	{
		char buffer[BUFFERSIZE];

		va_list args;
		va_start(args, format);
		vStringFormat(buffer, sizeof(buffer), format, args);
		va_end(args);

		result->SetTitle(buffer);
	}

	return(result);
}

// label can be NULL.
// The subsection is returned so it can have further details added when required.
CCSFSection *CCSFSection::AddFrameSubSection(	const char *label,
												unsigned int xpos, unsigned int ypos,
												unsigned int width, unsigned int height,
												CCSFSection::FontType fontType,
												CCSFSection::Colour labelTextColour,
												const char *iconPath)
{
	CCSFSection *result = AddSubSection();

	result->SetTitleFormat("Frame(\"%s\",%u,%u,%u,%u)", NULL == label ? "" : label, xpos, ypos, width, height);
	result->AddTitleOnlySubSectionFormat("TextColor(%s)", getColourString(labelTextColour));
	result->AddTitleOnlySubSectionFormat("Font(%u);", (unsigned int)(fontType));

	if ((NULL != iconPath) && (0 < strlen(iconPath)))
	{
		result->AddTitleOnlySubSectionFormat("Icon(\"%s\");", iconPath);
	}

	return(result);
}

// label can be NULL.
// The subsection is returned so it can have further details added when required.
CCSFSection *CCSFSection::AddButtonSubSection(	const char *label,
												unsigned int xpos, unsigned int ypos,
												unsigned int width, unsigned int height,
												CCSFSection::FontType fontType,
												CCSFSection::Colour labelTextColour,
												const char *iconPath,
												CCSFSection::ButtonType buttonType,
												unsigned int discOrPageNumber)
{
	CCSFSection *result = AddSubSection();

	unsigned int buttonNumber = m_pEnvironment->GetAndIncrementNextButtonNumber();

	result->SetTitleFormat("Button(\"%s\",%u,%u,%u,%u,%u) /* Button %u */",
															NULL == label ? "" : label, xpos, ypos, width, height,
															buttonNumber, buttonNumber);

	result->AddTitleOnlySubSectionFormat("ActiveUnselectedColor(LightGray,%s);", getColourString(labelTextColour));
	result->AddTitleOnlySubSectionFormat("ActiveSelectedColor(LightGray,%s);", getColourString(labelTextColour));
	result->AddTitleOnlySubSectionFormat("Font(%u);", (unsigned int)(fontType));
	result->AddTitleOnlySubSectionFormat("ActiveIcons(\"%s\");", iconPath);

	CCSFSection *actionSection = result->AddSubSection();
	actionSection->SetTitle("Action()");

	switch(buttonType)
	{
	default:
	case(CCSFSection::BUTTON_PLAYDISC):
		{
			// Disc button.

			actionSection->AddTitleOnlySubSection(m_pEnvironment->GetBMDisc()->GetAliasString());

			// Number buttons, with delays between two identical digits.

			CUnsignedBaseTenNumber ubtnDiscNumber(discOrPageNumber);
			unsigned int numDigits = ubtnDiscNumber.GetNumberOfDigits();
			unsigned int currentDigit;
			unsigned int previousDigit;

			previousDigit = ubtnDiscNumber.GetDigit(0);
			actionSection->AddTitleOnlySubSection(m_pEnvironment->GetBMNumber(previousDigit)->GetAliasString());

			for (unsigned int i = 1; i < numDigits; i++)
			{
				currentDigit = ubtnDiscNumber.GetDigit(i);

				if (currentDigit == previousDigit)
				{
					actionSection->AddTitleOnlySubSection("Delay(200);");
				}

				actionSection->AddTitleOnlySubSection(m_pEnvironment->GetBMNumber(currentDigit)->GetAliasString());

				previousDigit = currentDigit;
			}

			// Enter button

			actionSection->AddTitleOnlySubSection(m_pEnvironment->GetBMEnter()->GetAliasString());

			// Jump back to the main CD panel.

			actionSection->AddTitleOnlySubSection("JumpDevicePanel(\"CD\", \"Main\");");
		}
		break;
	case(CCSFSection::BUTTON_JUMPARTISTPAGE):
		{
			actionSection->AddTitleOnlySubSectionFormat("JumpDevicePanel(\"this\", \"A%u\");", discOrPageNumber);
		}
		break;
	case(CCSFSection::BUTTON_JUMPDISCPAGE):
		{
			actionSection->AddTitleOnlySubSectionFormat("JumpDevicePanel(\"this\", \"D%u\");", discOrPageNumber);
		}
		break;
	case(CCSFSection::BUTTON_JUMPALPHABET):
		{
			actionSection->AddTitleOnlySubSection("JumpDevicePanel(\"this\", \"Alphabet\");");
		}
		break;
	case(CCSFSection::BUTTON_JUMPCREDITS):
		{
			actionSection->AddTitleOnlySubSection("JumpDevicePanel(\"this\", \"Credits\");");
		}
		break;
	}

	return(result);
}


// Returns NULL if there is no matching subsection.
CCSFSection *CCSFSection::FindSubSectionByName(const char *name) const
{
	CCSFSection *result = NULL;

	const char *candidateName;

	if (m_bHasSubBody)
	{
		for (const_subIterator iter = subBegin(); subEnd() != iter; iter++)
		{
			candidateName = (*iter)->GetTitle();

			if (((NULL == name) && (NULL == candidateName))
			||  ((NULL != name) && (NULL != candidateName) && (0 == _stricmp(name, candidateName))))
			{
				result = (*iter);
				break;
			}
		}
	}

	return(result);
}

CCSFSection *CCSFSection::FindSubSectionByNamePrefix(const char *namePrefix) const
{
	CCSFSection *result = NULL;

	const char *candidateName;

	if (m_bHasSubBody)
	{
		size_t namePrefixLen = strlen(namePrefix);

		for (const_subIterator iter = subBegin(); subEnd() != iter; iter++)
		{
			candidateName = (*iter)->GetTitle();

			if (((NULL == namePrefix) && (NULL == candidateName))
			||  ((NULL != namePrefix) && (NULL != candidateName) && (0 == _strnicmp(namePrefix, candidateName, namePrefixLen))))
			{
				result = (*iter);
				break;
			}
		}
	}

	return(result);
}

CCSFSection *CCSFSection::FindButtonSubInAnySub(const char *buttonName, ButtonSearchType searchType) const
{
	CCSFSection *result = NULL;

	CCSFSection *pDirectSub;
	CCSFSection *pCandidateSub;

	if (m_bHasSubBody && (NULL != buttonName))
	{
		const_subIterator directSubIter;
		const_subIterator buttonLevelSubIter;

		for (directSubIter = this->subBegin(); (NULL == result) && (this->subEnd() != directSubIter); directSubIter++)
		{
			pDirectSub = (*directSubIter);

			if (pDirectSub->m_bHasSubBody)
			{
				for (buttonLevelSubIter = pDirectSub->subBegin(); pDirectSub->subEnd() != buttonLevelSubIter; buttonLevelSubIter++)
				{
					pCandidateSub = (*buttonLevelSubIter);

					if (pCandidateSub->MatchesButtonType(buttonName, searchType))
					{
						result = pCandidateSub;
						break;
					}
				}
			}
		}
	}

	return(result);
}

bool CCSFSection::MatchesButtonType(const char *buttonName, ButtonSearchType searchType) const
{
	bool bResult = false;

	if (NULL != m_title)
	{
		char *actualButtonName;

		// Is it a button? Get its name if it is.
		if (ScanForButton(&actualButtonName, NULL))
		{
			// Does it have the name we're looking for?
			if (0 == _stricmp(buttonName, actualButtonName))
			{
				bResult = true; // Unless it doesn't match with one of the flags...

				switch(searchType)
				{
				default:
					{
						printf("WARNING: Internal error, unknown button search type\n");
						bResult = false;
					}
					break;

				case(BUTTONSEARCH_ANY):
					break;

				case(BUTTONSEARCH_NOACTION):
					{
						if (NULL != FindSubSectionByName("Action()"))
						{
							bResult = false;
						}
					}
					break;

				case(BUTTONSEARCH_ACTIONWITHOUTALIAS):
					{
						CCSFSection *pActionSection = FindSubSectionByName("Action()");

						if (NULL == pActionSection)
						{
							bResult = false;
						}
						else if (NULL != pActionSection->FindSubSectionByNamePrefix("AliasDeviceButton("))
						{
							bResult = false;
						}
					}
					break;
				}
			}

			delete [] actualButtonName;
		}
	}

	return(bResult);
}

void CCSFSection::RemoveAndDeleteSubSectionByPointer(CCSFSection *pSectionToFind)
{
	if (m_bHasSubBody)
	{
		m_subSections.remove(pSectionToFind); // Remove by value.
		delete pSectionToFind;

		// Do *not* update m_bHasSubBody. If m_subSections is now empty we have an empty subbody.
	}
}

const char *CCSFSection::getColourString(CCSFSection::Colour kula) const
{
	const char *result;

	switch(kula)
	{
	default:
	case(CCSFSection::COLOUR_BLACK):
		result = "Black";
		break;
	case(CCSFSection::COLOUR_DARKGRAY):
		result = "DarkGray";
		break;
	case(CCSFSection::COLOUR_LIGHTGRAY):
		result = "LightGray";
		break;
	case(CCSFSection::COLOUR_WHITE):
		result = "White";
		break;
	}

	return(result);
}

void CCSFSection::updateNextButtonNumber()
{
	unsigned int buttonNumber;

	if (ScanForButton(NULL, &buttonNumber))
	{
		m_pEnvironment->UpdateNextButtonNumber(buttonNumber);
	}
}

// Returns true if a button was parsed and the output arguments are valid.
// If returns true and pNameBuffer not NULL on input, *pNameBuffer is the name and should be delete[]'d.
// If returns true and pNumber not NULL on input, *pNumber is the number.
bool CCSFSection::ScanForButton(char **pNameBuffer, unsigned int *pNumber) const
{
	bool bResult = false;

	// Required for failure cleanup code.
	if (NULL != pNameBuffer)
	{
		(*pNameBuffer) = NULL;
	}

	const char *sectionTitle = m_title;

	if (NULL != sectionTitle)
	{
		const char *buttonPrefix = "Button(";
		size_t buttonPrefixLength = strlen(buttonPrefix);

		if (0 == _strnicmp(sectionTitle, buttonPrefix, buttonPrefixLength))
		{
			sectionTitle += buttonPrefixLength;

			size_t buttonArgsLen = strcspn(sectionTitle, ")");

			if (strlen(sectionTitle) != buttonArgsLen)
			{
				buttonArgsLen++;

				char *buttonArgsBuffer = new char[buttonArgsLen];
				stringCopy(buttonArgsBuffer, sectionTitle, buttonArgsLen);

				const char *bufPtr = buttonArgsBuffer;
				const char *delimiters = ",";
				size_t tokenBufferSize = buttonArgsLen;
				char *tokenBuffer = new char[tokenBufferSize];

				// Button name is the first argument
				// Button number is the sixth argument
				if (TokenizeEx(&bufPtr, delimiters, tokenBuffer, tokenBufferSize, false))
				{
					if (NULL != pNameBuffer)
					{
						if ((strlen(tokenBuffer) > 1) && ('"' == tokenBuffer[0]) && ('"' == tokenBuffer[strlen(tokenBuffer) - 1]))
						{
							// Strip surrounding quotes from the button name.
							(*pNameBuffer) = new char[strlen(tokenBuffer) - 1];
							stringCopy(*pNameBuffer, tokenBuffer + 1, strlen(tokenBuffer) - 1);
						}
						else
						{
							(*pNameBuffer) = new char[strlen(tokenBuffer) + 1];
							strcpy(*pNameBuffer, tokenBuffer);
						}
					}

					if (TokenizeEx(&bufPtr, delimiters, tokenBuffer, tokenBufferSize, false)	\
					&&  TokenizeEx(&bufPtr, delimiters, tokenBuffer, tokenBufferSize, false)	\
					&&  TokenizeEx(&bufPtr, delimiters, tokenBuffer, tokenBufferSize, false)	\
					&&  TokenizeEx(&bufPtr, delimiters, tokenBuffer, tokenBufferSize, false)	\
					&&  TokenizeEx(&bufPtr, delimiters, tokenBuffer, tokenBufferSize, false)	)
					{
						unsigned int buttonNumber = 0;
						
						if ((1 == sscanf(tokenBuffer, "%u", &buttonNumber)) && (0 != buttonNumber))
						{
							if (NULL != pNumber)
							{
								(*pNumber) = buttonNumber;
							}

							bResult = true; // Looks like a button to me.
						}
					}
				}

				delete [] tokenBuffer;
				delete [] buttonArgsBuffer;
			}
		}
	}

	// Failure cleanup code.
	if (!bResult)
	{
		// If we're returning false, free partial results.

		if ((NULL != pNameBuffer) && (NULL != (*pNameBuffer)))
		{
			delete [] (*pNameBuffer);
			(*pNameBuffer) = NULL;
		}
	}

	return(bResult);
}

// Returns true if a device was parsed and the output arguments are valid.
// If returns true and pNameBuffer not NULL on input, *pNameBuffer is the name and should be delete[]'d.
bool CCSFSection::ScanForDevice(char **pNameBuffer) const
{
	bool bResult = false;

	// Required for failure cleanup code.
	if (NULL != pNameBuffer)
	{
		(*pNameBuffer) = NULL;
	}

	const char *sectionTitle = m_title;

	if (NULL != sectionTitle)
	{
		const char *devicePrefix = "Device(";
		size_t devicePrefixLength = strlen(devicePrefix);

		if (0 == _strnicmp(sectionTitle, devicePrefix, devicePrefixLength))
		{
			sectionTitle += devicePrefixLength;

			size_t deviceArgsLen = strcspn(sectionTitle, ")");

			if (strlen(sectionTitle) != deviceArgsLen)
			{
				deviceArgsLen++;

				char *deviceArgsBuffer = new char[deviceArgsLen];
				stringCopy(deviceArgsBuffer, sectionTitle, deviceArgsLen);

				const char *bufPtr = deviceArgsBuffer;
				const char *delimiters = ",";
				size_t tokenBufferSize = deviceArgsLen;
				char *tokenBuffer = new char[tokenBufferSize];

				// Device name is the first argument
				if (TokenizeEx(&bufPtr, delimiters, tokenBuffer, tokenBufferSize, false))
				{
					if (NULL != pNameBuffer)
					{
						if ((strlen(tokenBuffer) > 1) && ('"' == tokenBuffer[0]) && ('"' == tokenBuffer[strlen(tokenBuffer) - 1]))
						{
							// Strip surrounding quotes from the button name.
							(*pNameBuffer) = new char[strlen(tokenBuffer) - 1];
							stringCopy(*pNameBuffer, tokenBuffer + 1, strlen(tokenBuffer) - 1);
						}
						else
						{
							(*pNameBuffer) = new char[strlen(tokenBuffer) + 1];
							strcpy(*pNameBuffer, tokenBuffer);
						}
					}

					bResult = true; // Looks like a device to me.
				}

				delete [] tokenBuffer;
				delete [] deviceArgsBuffer;
			}
		}
	}

	// Failure cleanup code.
	if (!bResult)
	{
		// If we're returning false, free partial results.

		if ((NULL != pNameBuffer) && (NULL != (*pNameBuffer)))
		{
			delete [] (*pNameBuffer);
			(*pNameBuffer) = NULL;
		}
	}

	return(bResult);
}

// Returns true if a panel was parsed and the output arguments are valid.
// If returns true and pNameBuffer not NULL on input, *pNameBuffer is the name and should be delete[]'d.
bool CCSFSection::ScanForPanel(char **pNameBuffer) const
{
	bool bResult = false;

	// Required for failure cleanup code.
	if (NULL != pNameBuffer)
	{
		(*pNameBuffer) = NULL;
	}

	const char *sectionTitle = m_title;

	if (NULL != sectionTitle)
	{
		const char *panelPrefix = "Panel(";
		size_t panelPrefixLength = strlen(panelPrefix);

		if (0 == _strnicmp(sectionTitle, panelPrefix, panelPrefixLength))
		{
			sectionTitle += panelPrefixLength;

			size_t panelArgsLen = strcspn(sectionTitle, ")");

			if (strlen(sectionTitle) != panelArgsLen)
			{
				panelArgsLen++;

				char *panelArgsBuffer = new char[panelArgsLen];
				stringCopy(panelArgsBuffer, sectionTitle, panelArgsLen);

				const char *bufPtr = panelArgsBuffer;
				const char *delimiters = ",";
				size_t tokenBufferSize = panelArgsLen;
				char *tokenBuffer = new char[tokenBufferSize];

				// Panel name is the first argument
				if (TokenizeEx(&bufPtr, delimiters, tokenBuffer, tokenBufferSize, false))
				{
					if (NULL != pNameBuffer)
					{
						if ((strlen(tokenBuffer) > 1) && ('"' == tokenBuffer[0]) && ('"' == tokenBuffer[strlen(tokenBuffer) - 1]))
						{
							// Strip surrounding quotes from the button name.
							(*pNameBuffer) = new char[strlen(tokenBuffer) - 1];
							stringCopy(*pNameBuffer, tokenBuffer + 1, strlen(tokenBuffer) - 1);
						}
						else
						{
							(*pNameBuffer) = new char[strlen(tokenBuffer) + 1];
							strcpy(*pNameBuffer, tokenBuffer);
						}
					}

					bResult = true; // Looks like a device to me.
				}

				delete [] tokenBuffer;
				delete [] panelArgsBuffer;
			}
		}
	}

	// Failure cleanup code.
	if (!bResult)
	{
		// If we're returning false, free partial results.

		if ((NULL != pNameBuffer) && (NULL != (*pNameBuffer)))
		{
			delete [] (*pNameBuffer);
			(*pNameBuffer) = NULL;
		}
	}

	return(bResult);
}

/////////////////////////////////////////////////////////////////////////
