
#ifndef __CSFSECTION_H_
#define __CSFSECTION_H_

#include "SimpleTextFile.h"


/////////////////////////////////////////////////////////////////////////

class CCSFButtonMemo
{
public:
	CCSFButtonMemo(const char *device, const char *panel, const char *name, unsigned int number);
	virtual ~CCSFButtonMemo(void);

	const char *GetDevice(void) const;
	const char *GetPanel(void) const;
	const char *GetName(void) const;
	unsigned int GetNumber(void) const;

	const char *GetAliasString(void) const;

protected:
	void generateAliasString(void);

protected:
	char *m_device;
	char *m_panel;
	char *m_name;
	unsigned int m_number;

	char *m_aliasString;
};

/////////////////////////////////////////////////////////////////////////

class CCSFEnvironment
{
public:
	CCSFEnvironment(void);
	virtual ~CCSFEnvironment(void);

	unsigned int GetAndIncrementNextButtonNumber(void);
	void UpdateNextButtonNumber(unsigned int foundButtonNumber);

	CCSFButtonMemo *GetBMDisc(void) const;
	CCSFButtonMemo *GetBMEnter(void) const;
	CCSFButtonMemo *GetBMNumber(unsigned int number) const;

	void SetBMDisc(CCSFButtonMemo *pBM);
	void SetBMEnter(CCSFButtonMemo *pBM);
	void SetBMNumber(unsigned int number, CCSFButtonMemo *pBM);


protected:
	unsigned int m_nextButtonNumber;

	CCSFButtonMemo *m_pBMDisc;
	CCSFButtonMemo *m_pBMEnter;
	CCSFButtonMemo *m_pBMNumbers[10];
};

/////////////////////////////////////////////////////////////////////////

class CCSFSection
{
public:
	typedef list<CCSFSection *>::iterator subIterator;
	typedef list<CCSFSection *>::const_iterator const_subIterator;

	enum FontType
	{
		FONT_0  = 0,
		FONT_8  = 8,
		FONT_10 = 10
	};

	enum Colour
	{
		COLOUR_BLACK,
		COLOUR_DARKGRAY,
		COLOUR_LIGHTGRAY,
		COLOUR_WHITE
	};

	enum ButtonType
	{
		BUTTON_NONE,
		BUTTON_PLAYDISC,
		BUTTON_JUMPARTISTPAGE,
		BUTTON_JUMPDISCPAGE,
		BUTTON_JUMPALPHABET,
		BUTTON_JUMPCREDITS
	};

	enum ButtonSearchType
	{
		BUTTONSEARCH_ANY,
		BUTTONSEARCH_NOACTION,
		BUTTONSEARCH_ACTIONWITHOUTALIAS
	};

public:
	CCSFSection(CCSFSection *pSectionParent);
	virtual ~CCSFSection(void);

	CCSFEnvironment *GetEnvironment(void);
	CCSFSection *CCSFSection::GetParent(void);
	const CCSFSection *CCSFSection::GetParent(void) const;

	const char *GetTitle(void);
	void SetTitle(const char *title);
	void SetTitleFormat(const char *format, ...);

	subIterator subBegin(void);
	subIterator subEnd(void);
	const_subIterator subBegin(void) const;
	const_subIterator subEnd(void) const;

	bool ReadNextSection(CSimpleTextFile::const_lineIterator &lineIter, const CSimpleTextFile::const_lineIterator &lineIterEnd);

	bool Write(FILE *pfOutputCSF, int indentLevel = 0) const;

	// The subsection is returned so it can have further details added when required.
	// Titles, formats, and labels can be NULL.
	CCSFSection *AddSubSection(void);
	CCSFSection *AddTitleOnlySubSection(const char *subTitle);
	CCSFSection *AddTitleOnlySubSectionFormat(const char *format, ...);
	CCSFSection *AddFrameSubSection(const char *label,
									unsigned int xpos, unsigned int ypos,
									unsigned int width, unsigned int height,
									CCSFSection::FontType fontType,
									CCSFSection::Colour labelTextColour,
									const char *iconPath);
	CCSFSection *AddButtonSubSection(	const char *label,
										unsigned int xpos, unsigned int ypos,
										unsigned int width, unsigned int height,
										CCSFSection::FontType fontType,
										CCSFSection::Colour labelTextColour,
										const char *iconPath,
										CCSFSection::ButtonType buttonType,
										unsigned int discOrPageNumber);

	// Returns NULL if there is no matching subsection.
	CCSFSection *FindSubSectionByName(const char *name) const;
	CCSFSection *FindSubSectionByNamePrefix(const char *namePrefix) const;
	CCSFSection *FindButtonSubInAnySub(const char *buttonName, ButtonSearchType searchType) const;

	bool MatchesButtonType(const char *buttonName, ButtonSearchType searchType) const;

	// Returns true if a button was parsed and the output arguments are valid.
	// If returns true and pNameBuffer not NULL on input, *pNameBuffer is the name and should be delete[]'d.
	// If returns true and pNumber not NULL on input, *pNumber is the number.
	bool ScanForButton(char **pNameBuffer, unsigned int *pNumber) const;

	// Returns true if a device was parsed and the output arguments are valid.
	// If returns true and pNameBuffer not NULL on input, *pNameBuffer is the name and should be delete[]'d.
	bool ScanForDevice(char **pNameBuffer) const;

	// Returns true if a panel was parsed and the output arguments are valid.
	// If returns true and pNameBuffer not NULL on input, *pNameBuffer is the name and should be delete[]'d.
	bool ScanForPanel(char **pNameBuffer) const;

	void RemoveAndDeleteSubSectionByPointer(CCSFSection *pSectionToFind);

protected:
	void indentLine(FILE *pfOutputCSF, int indentLevel) const;
	const char *getColourString(CCSFSection::Colour kula) const;
	void updateNextButtonNumber(void);

protected:
	CCSFSection *m_pSectionParent;
	char *m_title;
	bool m_bHasSubBody;
	CCSFEnvironment *m_pEnvironment;

	list<CCSFSection *> m_subSections;
};

#endif
