
#include "stdafx.h"

#include "Discography.h"

/////////////////////////////////////////////////////////////////////////

CDisc::CDisc(unsigned int num, const char *title)
{
	_ASSERTE(0 != num);
	_ASSERTE(NULL != title);

	m_num = num;
	m_title = new char[strlen(title) + 1];
	strcpy(m_title, title);
}

CDisc::~CDisc(void)
{
	if (NULL != m_title)
	{
		delete [] m_title;
		m_title = NULL;
	}
}

unsigned int CDisc::GetNum(void) const
{
	return(m_num);
}

const char *CDisc::GetTitle(void) const
{
	return(m_title);
}

#ifdef _DEBUG
void CDisc::DebugDumpToFile(FILE *pfOutput) const
{
	fprintf(pfOutput, "\t%03u: %s\n", m_num, m_title);
}
#endif

/////////////////////////////////////////////////////////////////////////

CDiscList::CDiscList(void)
{
}

CDiscList::~CDiscList(void)
{
	while(! m_discs.empty() )
	{
		delete m_discs.back();
		m_discs.pop_back();
	}
}

void CDiscList::AddTail(int num, const char *title)
{
	CDisc *pDisc = new CDisc(num, title);

	m_discs.push_back(pDisc);
}

CDiscList::size_type CDiscList::GetTotal(void) const
{
	return(m_discs.size());
}

CDiscList::iterator CDiscList::begin(void)
{
	return(m_discs.begin());
}

CDiscList::iterator CDiscList::end(void)
{
	return(m_discs.end());
}

CDiscList::const_iterator CDiscList::begin(void) const
{
	return(m_discs.begin());
}

CDiscList::const_iterator CDiscList::end(void) const
{
	return(m_discs.end());
}

#ifdef _DEBUG
void CDiscList::DebugDumpToFile(FILE *pfOutput) const
{
	list<CDisc *>::const_iterator iter;

	for (iter = m_discs.begin(); m_discs.end() != iter; iter++)
	{
		(*iter)->DebugDumpToFile(pfOutput);
	}
}
#endif

/////////////////////////////////////////////////////////////////////////

CArtist::CArtist(const char *name)
{
	m_name = new char[strlen(name) + 1];
	strcpy(m_name, name);

	m_shortName = NULL;
}

CArtist::~CArtist(void)
{
	if (NULL != m_name)
	{
		delete [] m_name;
		m_name = NULL;
	}

	SetShortName(NULL);
}

void CArtist::SetShortName(const char *shortName)
{
	if (NULL != m_shortName)
	{
		delete [] m_shortName;
		m_shortName = NULL;
	}

	if (NULL != shortName)
	{
		m_shortName = new char[strlen(shortName) + 1];
		strcpy(m_shortName, shortName);
	}
}

const char *CArtist::GetName(void) const
{
	return(m_name);
}

const char *CArtist::GetShortName(void) const
{
	return(NULL != m_shortName ? m_shortName : m_name);
}

CDiscList *CArtist::GetDiscList(void)
{
	return(&m_discList);
}

const CDiscList *CArtist::GetDiscList(void) const
{
	return(&m_discList);
}

#ifdef _DEBUG
void CArtist::DebugDumpToFile(FILE *pfOutput) const
{
	fprintf(pfOutput, "%s\n", m_name);

	m_discList.DebugDumpToFile(pfOutput);
}
#endif

/////////////////////////////////////////////////////////////////////////

CArtistList::CArtistList(void)
{
}

CArtistList::~CArtistList(void)
{
	while(! m_artists.empty() )
	{
		delete m_artists.front();
		m_artists.pop_front();
	}
}

CArtist *CArtistList::FindElseAdd(const char *name)
{
	CArtist *pCAResult = NULL;

	_ASSERTE(NULL != name);

	list<CArtist *>::iterator iter;

	for (iter = m_artists.begin(); iter != m_artists.end(); iter++)
	{
		if (0 == _stricmp(name, (*iter)->GetName()))
		{
			pCAResult = (*iter);
		}
	}

	if (NULL == pCAResult)
	{
		pCAResult = new CArtist(name);
		m_artists.push_front(pCAResult);
	}

	return(pCAResult);
}

// I tried for two and a half hours to find a way to get list::sort to do what I wanted.
// I gave up.
// The documentation is TERRIBLE.
// C++ templates are not my favourite thing, either. Especially when implemented differently...
// VC++'97 is definitely not my favourite thing.
// Give me functional programming any day!

int pArtistStricmp(const void *pLeft, const void *pRight)
{
	CArtist *pLeftArtist  = *(CArtist**)pLeft;
	CArtist *pRightArtist = *(CArtist**)pRight;

	return(stricmp(pLeftArtist->GetName(), pRightArtist->GetName()));
}

void CArtistList::Sort(void)
{
	int numArtists = m_artists.size();

	// To get around the C++ / VC++'97 fuck up which means that
	// "CArtist *artistsArray[] = new (CArtist *)[numArtists];"
	// will not compile. Sigh.
	typedef CArtist * PCArtist;

	PCArtist *artistsArray = new PCArtist[numArtists];

	// Suck list out to array of (CArtist *)'s for sorting.
	for (int i=0; i < numArtists; i++)
	{
		_ASSERTE(!m_artists.empty());

		artistsArray[i] = m_artists.front();
		m_artists.pop_front();
	}

	 _ASSERTE(m_artists.empty());

	// Sort array.
	_ASSERTE(sizeof(CArtist *) == sizeof(PCArtist)); // just in case the type changes...
	qsort(artistsArray, numArtists, sizeof(CArtist *), pArtistStricmp);

	// Rebuild list.
	for (/*int [SMMFD, Microsoft]*/i=0; i < numArtists; i++)
	{
		m_artists.push_back(artistsArray[i]);
	}

	delete [] artistsArray;
}

CArtistList::iterator CArtistList::begin(void)
{
	return(m_artists.begin());
}

CArtistList::iterator CArtistList::end(void)
{
	return(m_artists.end());
}

CArtistList::const_iterator CArtistList::begin(void) const
{
	return(m_artists.begin());
}

CArtistList::const_iterator CArtistList::end(void) const
{
	return(m_artists.end());
}

#ifdef _DEBUG
void CArtistList::DebugDumpToFile(FILE *pfOutput) const
{
	list<CArtist *>::const_iterator iter;

	fprintf(pfOutput, "-- Debug Dump Start --\n");

	for (iter = m_artists.begin(); m_artists.end() != iter; iter++)
	{
		(*iter)->DebugDumpToFile(pfOutput);
	}

	fprintf(pfOutput, "-- Debug Dump End --\n");
}
#endif
