
#ifndef __DISCOGRAPHY_H_
#define __DISCOGRAPHY_H_

/////////////////////////////////////////////////////////////////////////

class CDisc
{
public:
	CDisc(unsigned int num, const char *title);
	virtual ~CDisc(void);

	unsigned int GetNum(void) const;
	const char *GetTitle(void) const;

#ifdef _DEBUG
	void DebugDumpToFile(FILE *pfOutput) const;
#endif

private:
	unsigned int m_num;
	char *m_title;
};

/////////////////////////////////////////////////////////////////////////

class CDiscList
{
public:
	CDiscList(void);
	virtual ~CDiscList(void);

	void AddTail(int num, const char *title);

	typedef list<CDisc *>::size_type size_type;

	size_type GetTotal(void) const;

	typedef list<CDisc *>::iterator iterator;
	typedef list<CDisc *>::const_iterator const_iterator;

	iterator begin(void);
	iterator end(void);
	const_iterator begin(void) const;
	const_iterator end(void) const;

#ifdef _DEBUG
	void DebugDumpToFile(FILE *pfOutput) const;
#endif

private:
	list<CDisc *> m_discs;
};

/////////////////////////////////////////////////////////////////////////

class CArtist
{
public:
	CArtist(const char *name);
	virtual ~CArtist(void);

	void SetShortName(const char *shortName);

	const char *GetName(void) const;
	const char *GetShortName(void) const;

	CDiscList *GetDiscList(void);
	const CDiscList *GetDiscList(void) const;

#ifdef _DEBUG
	void DebugDumpToFile(FILE *pfOutput) const;
#endif

private:
	char *m_name;
	char *m_shortName;
	CDiscList m_discList;
};

/////////////////////////////////////////////////////////////////////////

class CArtistList
{
public:
	CArtistList(void);
	virtual ~CArtistList(void);

	CArtist *FindElseAdd(const char *name);

	void Sort(void);

	typedef list<CArtist *>::iterator iterator;
	typedef list<CArtist *>::const_iterator const_iterator;

	iterator begin(void);
	iterator end(void);
	const_iterator begin(void) const;
	const_iterator end(void) const;

#ifdef _DEBUG
	void DebugDumpToFile(FILE *pfOutput) const;
#endif

private:
	list<CArtist *> m_artists;
};

int pArtistStricmp(const void *pLeft, const void *pRight);

#endif

