
#ifndef __PANELLINES_H_
#define __PANELLINES_H_

#include "PanelList.h"

/////////////////////////////////////////////////////////////////////////

class CPLBlank : public CPanelLine
{
public:
	CPLBlank(void);
	virtual ~CPLBlank(void);

	virtual void AddToPanelCSFSection(CCSFSection *pPanelCSFSection, unsigned int ypos) const;
};

/////////////////////////////////////////////////////////////////////////

class CPLTwoArtists : public CPanelLine
{
public:
	CPLTwoArtists(const CArtist *pArtistOne, unsigned int artistOneDiscPanelNumber,
					const CArtist *pArtistTwo, unsigned int artistTwoDiscPanelNumber);
	virtual ~CPLTwoArtists(void);

	virtual void AddToPanelCSFSection(CCSFSection *pPanelCSFSection, unsigned int ypos) const;

protected:
	const CArtist *m_pArtistOne;
	const CArtist *m_pArtistTwo;
	unsigned int m_artistOneDiscPanelNumber;
	unsigned int m_artistTwoDiscPanelNumber;
};

/////////////////////////////////////////////////////////////////////////

class CPLOneArtistAndContinuation : public CPanelLine
{
public:
	CPLOneArtistAndContinuation(const CArtist *pArtistOne, unsigned int artistOneDiscPanelNumber,
								unsigned int continuationPanelNumber);
	virtual ~CPLOneArtistAndContinuation(void);

	virtual void AddToPanelCSFSection(CCSFSection *pPanelCSFSection, unsigned int ypos) const;

protected:
	const CArtist *m_pArtistOne;
	unsigned int m_artistOneDiscPanelNumber;
	unsigned int m_continuationPanelNumber;
};

/////////////////////////////////////////////////////////////////////////

class CPLArtistsLetterHeading : public CPanelLine
{
public:
	CPLArtistsLetterHeading(const char letter);
	virtual ~CPLArtistsLetterHeading(void);

	virtual void AddToPanelCSFSection(CCSFSection *pPanelCSFSection, unsigned int ypos) const;

protected:
	char m_letter;
};

/////////////////////////////////////////////////////////////////////////

class CPLArtistsLetterContinuedHeading : public CPanelLine
{
public:
	CPLArtistsLetterContinuedHeading(const char letter, unsigned int previousLetterPanelNumber);
	virtual ~CPLArtistsLetterContinuedHeading(void);

	virtual void AddToPanelCSFSection(CCSFSection *pPanelCSFSection, unsigned int ypos) const;

protected:
	char m_letter;
	unsigned int m_previousLetterPanelNumber;
};

/////////////////////////////////////////////////////////////////////////

class CPLArtistHeading : public CPanelLine
{
public:
	CPLArtistHeading(const CArtist *pArtist, unsigned int artistPanelNumber);
	virtual ~CPLArtistHeading(void);

	virtual void AddToPanelCSFSection(CCSFSection *pPanelCSFSection, unsigned int ypos) const;

protected:
	const CArtist *m_pArtist;
	unsigned int m_artistPanelNumber;
};

/////////////////////////////////////////////////////////////////////////

class CPLArtistContinuedHeading : public CPanelLine
{
public:
	CPLArtistContinuedHeading(const CArtist *pArtist, unsigned int previousPanelNumber);
	virtual ~CPLArtistContinuedHeading(void);

	virtual void AddToPanelCSFSection(CCSFSection *pPanelCSFSection, unsigned int ypos) const;

protected:
	const CArtist *m_pArtist;
	unsigned int m_previousPanelNumber;
};

/////////////////////////////////////////////////////////////////////////

class CPLOneDiscAndContinuation : public CPanelLine
{
public:
	CPLOneDiscAndContinuation(const CDisc *pDiscOne, unsigned int continuationPanelNumber);
	virtual ~CPLOneDiscAndContinuation(void);

	virtual void AddToPanelCSFSection(CCSFSection *pPanelCSFSection, unsigned int ypos) const;

protected:
	const CDisc *m_pDiscOne;
	unsigned int m_continuationPanelNumber;
};

/////////////////////////////////////////////////////////////////////////

class CPLOneDisc : public CPanelLine
{
public:
	CPLOneDisc(const CDisc *pDiscOne);
	virtual ~CPLOneDisc(void);

	virtual void AddToPanelCSFSection(CCSFSection *pPanelCSFSection, unsigned int ypos) const;

protected:
	const CDisc *m_pDiscOne;
};

/////////////////////////////////////////////////////////////////////////

class CPLTwoDiscs : public CPanelLine
{
public:
	CPLTwoDiscs(const CDisc *pDiscOne, const CDisc *pDiscTwo);
	virtual ~CPLTwoDiscs(void);

	virtual void AddToPanelCSFSection(CCSFSection *pPanelCSFSection, unsigned int ypos) const;

protected:
	const CDisc *m_pDiscOne;
	const CDisc *m_pDiscTwo;
};

/////////////////////////////////////////////////////////////////////////

#endif
