
#include "stdafx.h"

#include "Discography.h"
#include "PanelList.h"
#include "PanelLines.h"

/////////////////////////////////////////////////////////////////////////

CPanel::CPanel(char prefixChar, unsigned int panelNumber)
{
	m_prefixChar = prefixChar;
	m_panelNumber = panelNumber;

	m_msgLeft = NULL;
	m_msgLeftLinkType = CCSFSection::BUTTON_NONE;
	m_msgLeftLinkNumber = 0;

	m_msgRight = NULL;
	m_msgRightLinkType = CCSFSection::BUTTON_NONE;
	m_msgRightLinkNumber = 0;
	
	m_numFreeLines = MAX_LINES;
}

CPanel::~CPanel(void)
{
	SetMsgLeft(CCSFSection::BUTTON_NONE, 0, NULL);
	SetMsgRight(CCSFSection::BUTTON_NONE, 0, NULL);

	while (! m_panelLines.empty() )
	{
		delete m_panelLines.back();
		m_panelLines.pop_back();
	}
}

char CPanel::GetPrefixChar(void)
{
	return(m_prefixChar);
}

unsigned int CPanel::GetNumber(void)
{
	return(m_panelNumber);
}

void CPanel::SetMsgLeft(CCSFSection::ButtonType linkType, unsigned int linkNumber, const char *msg, ...)
{
	if (NULL != m_msgLeft)
	{
		delete [] m_msgLeft;
		m_msgLeft = NULL;
	}

	m_msgLeftLinkType = linkType;
	m_msgLeftLinkNumber = linkNumber;

	if (NULL != msg)
	{
		// Oh for a vprintf which allocated the buffer! GOD DAMMIT.
		char tempBuffer[BUFFERSIZE];

		va_list args;
		va_start(args, msg);
		vStringFormat(tempBuffer, sizeof(tempBuffer), msg, args);
		va_end(args);

		m_msgLeft = new char[strlen(tempBuffer) + 1];
		strcpy(m_msgLeft, tempBuffer);
	}
}

void CPanel::SetMsgRight(CCSFSection::ButtonType linkType, unsigned int linkNumber, const char *msg, ...)
{
	if (NULL != m_msgRight)
	{
		delete [] m_msgRight;
		m_msgRight = NULL;
	}

	m_msgRightLinkType = linkType;
	m_msgRightLinkNumber = linkNumber;

	if (NULL != msg)
	{
		char tempBuffer[BUFFERSIZE];

		va_list args;
		va_start(args, msg);
		vStringFormat(tempBuffer, sizeof(tempBuffer), msg, args);
		va_end(args);

		m_msgRight = new char[strlen(tempBuffer) + 1];
		strcpy(m_msgRight, tempBuffer);
	}
}

unsigned int CPanel::FreeLines(void) const
{
	return(m_numFreeLines);
}

bool CPanel::IsEmpty(void) const
{
	return(CPanel::MAX_LINES == m_numFreeLines);
}

void CPanel::AddPanelLine(CPanelLine *pPanelLine)
{
	if (0 >= m_numFreeLines)
	{
		printf("WARNING: Internal error. Line added to full panel ignored.\n");
		delete pPanelLine;
	}
	else
	{
		m_panelLines.push_back(pPanelLine);
		m_numFreeLines--;
	}
}

void CPanel::AddToDeviceCSFSection(CCSFSection *pDeviceCSFSection) const
{
	CCSFSection *pOurNewPanel;

	pOurNewPanel = pDeviceCSFSection->AddSubSection();

	pOurNewPanel->SetTitleFormat("Panel(\"%c%u\")", (int)m_prefixChar, m_panelNumber);

	pOurNewPanel->AddTitleOnlySubSection("Font(0)");
	pOurNewPanel->AddTitleOnlySubSection("BackColor(DarkGray);");
	pOurNewPanel->AddTitleOnlySubSection("TextColor(Black);");
	pOurNewPanel->AddTitleOnlySubSection("InactiveUnselectedColor(LightGray,DarkGray);");
	pOurNewPanel->AddTitleOnlySubSection("InactiveSelectedColor(DarkGray,Black);");
	pOurNewPanel->AddTitleOnlySubSection("ActiveUnselectedColor(LightGray,Black);");
	pOurNewPanel->AddTitleOnlySubSection("ActiveSelectedColor(LightGray,DarkGray);");

	unsigned int ypos = 0;

	list<CPanelLine *>::const_iterator iter;

	for (iter = m_panelLines.begin(); m_panelLines.end() != iter; iter++)
	{
		(*iter)->AddToPanelCSFSection(pOurNewPanel, ypos);

		ypos += 34;
	}

	ypos = CPanel::MAX_LINES * 34;

	if (CCSFSection::BUTTON_NONE == m_msgLeftLinkType)
	{
		pOurNewPanel->AddFrameSubSection(	NULL != m_msgLeft ? m_msgLeft : NULL,
											0, ypos, 120, 15,
											CCSFSection::FONT_8,
											CCSFSection::COLOUR_BLACK,
											"BMP\\FinalBarLeft.bmp");
	}
	else
	{
		pOurNewPanel->AddButtonSubSection(	NULL != m_msgLeft ? m_msgLeft : NULL,
											0, ypos, 120, 15,
											CCSFSection::FONT_8,
											CCSFSection::COLOUR_BLACK,
											"BMP\\FinalBarLeft.bmp",
											m_msgLeftLinkType,
											m_msgLeftLinkNumber);
	}

	if (CCSFSection::BUTTON_NONE == m_msgRightLinkType)
	{
		pOurNewPanel->AddFrameSubSection(	NULL != m_msgRight ? m_msgRight : NULL,
											120, ypos, 120, 15,
											CCSFSection::FONT_8,
											CCSFSection::COLOUR_BLACK,
											"BMP\\FinalBarRight.bmp");
	}
	else
	{
		pOurNewPanel->AddButtonSubSection(	NULL != m_msgRight ? m_msgRight : NULL,
											120, ypos, 120, 15,
											CCSFSection::FONT_8,
											CCSFSection::COLOUR_BLACK,
											"BMP\\FinalBarRight.bmp",
											m_msgRightLinkType,
											m_msgRightLinkNumber);
	}
}

/////////////////////////////////////////////////////////////////////////

CAlphaPanelLetter::CAlphaPanelLetter(char letter, unsigned int artistPanelNumber)
{
	m_letter = letter;
	m_artistPanelNumber = artistPanelNumber;
}

CAlphaPanelLetter::~CAlphaPanelLetter(void)
{
}

char CAlphaPanelLetter::GetArtistPanelLetter(void)
{
	return(m_letter);
}

unsigned int CAlphaPanelLetter::GetArtistPanelNumber(void)
{
	return(m_artistPanelNumber);
}

/////////////////////////////////////////////////////////////////////////

CAlphabetPanel::CAlphabetPanel(void)
{
	for (int i = 0; i < LETTER_ARRAY_SIZE; i++)
	{
		m_letterArray[i] = NULL;
	}
}

CAlphabetPanel::~CAlphabetPanel(void)
{
	for (int i = 0; i < LETTER_ARRAY_SIZE; i++)
	{
		if (NULL != m_letterArray[i])
		{
			delete m_letterArray[i];
			m_letterArray[i] = NULL;
		}
	}
}

void CAlphabetPanel::AddToDeviceCSFSection(CCSFSection *pDeviceCSFSection) const
{
	CCSFSection *pOurNewPanel;

	pOurNewPanel = pDeviceCSFSection->AddSubSection();

	pOurNewPanel->SetTitle("Panel(\"Alphabet\")");

	pOurNewPanel->AddTitleOnlySubSection("Font(0)");
	pOurNewPanel->AddTitleOnlySubSection("BackColor(DarkGray);");
	pOurNewPanel->AddTitleOnlySubSection("TextColor(Black);");
	pOurNewPanel->AddTitleOnlySubSection("InactiveUnselectedColor(LightGray,DarkGray);");
	pOurNewPanel->AddTitleOnlySubSection("InactiveSelectedColor(DarkGray,Black);");
	pOurNewPanel->AddTitleOnlySubSection("ActiveUnselectedColor(LightGray,Black);");
	pOurNewPanel->AddTitleOnlySubSection("ActiveSelectedColor(LightGray,DarkGray);");

	// Add alphabet buttons in fixed places.
	// (That way the user need not re-learn their positions if they add an artist with a new letter.)

	unsigned int xpos;
	unsigned int ypos;
	char letterString[2];
	letterString[1] = '\0';

	for (int i = 0; i < LETTER_ARRAY_SIZE; i++)
	{
		// x-pos
		if (i < 6)
		{
			xpos = 40 * i;
		}
		else if (i > 20)
		{
			xpos = 40 * (i - 21);
		}
		else
		{
			xpos = 50 * ((i - 6) % 5);
		}

		// y-pos
		if (i < 6)
		{
			ypos = 0;
		}
		else if (i < 11)
		{
			ypos = 40;
		}
		else if (i < 16)
		{
			ypos = 80;
		}
		else if (i < 21)
		{
			ypos = 120;
		}
		else
		{
			ypos = 160;
		}

		if (NULL != m_letterArray[i])
		{

			letterString[0] = m_letterArray[i]->GetArtistPanelLetter();

			pOurNewPanel->AddButtonSubSection(	letterString,
												xpos, ypos, 38, 35,
												CCSFSection::FONT_8,
												CCSFSection::COLOUR_WHITE,
												"BMP\\SmallRoundBlackButton.bmp",
												CCSFSection::BUTTON_JUMPARTISTPAGE,
												m_letterArray[i]->GetArtistPanelNumber());
		}
		else
		{
			// Blank button to stop things looking silly.
			pOurNewPanel->AddFrameSubSection(	".",
												xpos, ypos, 38, 35,
												CCSFSection::FONT_8,
												CCSFSection::COLOUR_DARKGRAY,
												"BMP\\SmallRoundBlackButton.bmp");
		}
	}

	// Lower-left version string.

	ypos = CPanel::MAX_LINES * 34;

	const char *progName = "Nudelist";
	char *fileVersion = getFileVersion();

	size_t verStringSize = strlen(progName) + 2;

	if (NULL != fileVersion)
	{
		verStringSize += strlen(fileVersion) + 1;
	}

	char *verString = new char[verStringSize];

	strcpy(verString, "[");
	strcat(verString, progName);

	if (NULL != fileVersion)
	{
		strcat(verString, " ");
		strcat(verString, fileVersion);
	}

	strcat(verString, "]");

	pOurNewPanel->AddFrameSubSection(	verString,
										0, ypos, 120, 15,
										CCSFSection::FONT_8,
										CCSFSection::COLOUR_BLACK,
										"BMP\\FinalBarLeft.bmp");

	if (NULL != fileVersion)
	{
		delete [] fileVersion;
		fileVersion = NULL;
	}

	delete [] verString;
	verString = NULL;

	// Lower-right credits button.
	
	pOurNewPanel->AddButtonSubSection(	"Credits",
										120, ypos, 120, 15,
										CCSFSection::FONT_8,
										CCSFSection::COLOUR_BLACK,
										"BMP\\FinalBarRight.bmp",
										CCSFSection::BUTTON_JUMPCREDITS,
										0);
}

void CAlphabetPanel::NoteArtistPanelNumber(const char letterToFile, unsigned int artistPanelNumber)
{
	// If there's no alphanumeric in the name, ignore it.
	if ('\0' != letterToFile)
	{
		// This relies on the machine using ASCII, or at least sequential letters. :-)

		int letterArrayIndex = LETTER_ARRAY_SIZE;

		if (isupper(letterToFile))
		{
			letterArrayIndex = (letterToFile - 'A') + LETTER_ARRAY_ALPHA_OFFSET;
		}
		else if (islower(letterToFile))
		{
			letterArrayIndex = (letterToFile - 'a') + LETTER_ARRAY_ALPHA_OFFSET;
		}
		else if ('#' == letterToFile)
		{
			letterArrayIndex = 0;
		}

		if (LETTER_ARRAY_SIZE > letterArrayIndex)
		{
			// If the letter has already been filed, ignore it.
			if (NULL == m_letterArray[letterArrayIndex])
			{
				m_letterArray[letterArrayIndex] = new CAlphaPanelLetter(letterToFile, artistPanelNumber);
			}
		}
	}
}

/////////////////////////////////////////////////////////////////////////

CCreditsPanel::CCreditsPanel(void)
{
}

CCreditsPanel::~CCreditsPanel(void)
{
}

void CCreditsPanel::AddToDeviceCSFSection(CCSFSection *pDeviceCSFSection) const
{
	CCSFSection *pOurNewPanel;

	pOurNewPanel = pDeviceCSFSection->AddSubSection();

	pOurNewPanel->SetTitle("Panel(\"Credits\")");

	pOurNewPanel->AddTitleOnlySubSection("Font(8)");
	pOurNewPanel->AddTitleOnlySubSection("BackColor(White);");
	pOurNewPanel->AddTitleOnlySubSection("TextColor(DarkGray);");
	pOurNewPanel->AddTitleOnlySubSection("InactiveUnselectedColor(LightGray,DarkGray);");
	pOurNewPanel->AddTitleOnlySubSection("InactiveSelectedColor(DarkGray,Black);");
	pOurNewPanel->AddTitleOnlySubSection("ActiveUnselectedColor(LightGray,Black);");
	pOurNewPanel->AddTitleOnlySubSection("ActiveSelectedColor(LightGray,White);");

	// Nudel logo

	pOurNewPanel->AddFrameSubSection(	NULL,
										37, 15, 167, 71,
										CCSFSection::FONT_0,
										CCSFSection::COLOUR_BLACK,
										"BMP\\NudelPronto.bmp");

	// Text

	pOurNewPanel->AddFrameSubSection(	"Nudelist by Leo 'Nudel' Davidson",
										10, 90, 221, 21,
										CCSFSection::FONT_10,
										CCSFSection::COLOUR_BLACK,
										NULL);

	pOurNewPanel->AddFrameSubSection(	"leo@ox.compsoc.net",
										100, 115, 131, 16,
										CCSFSection::FONT_8,
										CCSFSection::COLOUR_BLACK,
										NULL);

	pOurNewPanel->AddFrameSubSection(	"http:\\\\www.ox.compsoc.net\\~leo",
										30, 133, 200, 16,
										CCSFSection::FONT_8,
										CCSFSection::COLOUR_BLACK,
										NULL);

	pOurNewPanel->AddFrameSubSection(	"Free. C++ source code available. Enjoy!",
										10, 165, 221, 16,
										CCSFSection::FONT_8,
										CCSFSection::COLOUR_BLACK,
										NULL);

	// Lower-left buttons.

	unsigned int ypos = CPanel::MAX_LINES * 34;

	pOurNewPanel->AddButtonSubSection(	"Alphabet",
										0, ypos, 120, 15,
										CCSFSection::FONT_8,
										CCSFSection::COLOUR_BLACK,
										"BMP\\FinalBarLeft.bmp",
										CCSFSection::BUTTON_JUMPALPHABET,
										0);
	
	pOurNewPanel->AddFrameSubSection(	"[Nudel, I love you!]",
										120, ypos, 120, 15,
										CCSFSection::FONT_8,
										CCSFSection::COLOUR_BLACK,
										"BMP\\FinalBarRight.bmp");
}

/////////////////////////////////////////////////////////////////////////

CPanelList::CPanelList(void)
{
}

CPanelList::~CPanelList(void)
{
	while (! m_artistPanels.empty() )
	{
		delete m_artistPanels.back();
		m_artistPanels.pop_back();
	}

	while (! m_discPanels.empty() )
	{
		delete m_discPanels.back();
		m_discPanels.pop_back();
	}
}

void CPanelList::DoLayout(const CArtistList *pArtistList)
{
	m_artistPanels.push_back(new CPanel('A', 0));
	m_artistPanels.back()->SetMsgLeft(CCSFSection::BUTTON_NONE, 0, "[Artists Start]");

	m_discPanels.push_back(new CPanel('D', 0));
	m_discPanels.back()->SetMsgLeft(CCSFSection::BUTTON_NONE, 0, "[Discs Start]");

	doLayoutArtists(pArtistList);

	m_artistPanels.back()->SetMsgRight(CCSFSection::BUTTON_NONE, 0, "[Artists End]");
	m_discPanels.back()->SetMsgRight(CCSFSection::BUTTON_NONE, 0, "[Discs End]");
}

void CPanelList::doLayoutArtists(const CArtistList *pArtistList)
{
	const CArtist *pArtist;
	const CArtist *pArtistPrevious = NULL;

	unsigned int artistDiscPanelNumber = 0;
	unsigned int artistDiscPanelNumberPrevious = 0;

	bool bAddArtistsThisTime = false;

	CArtistList::const_iterator iter;
	CDiscList::size_type numTotalDiscs;

	char artistFirstInterestingLetterPrevious = '\0';
	char artistFirstInterestingLetter = '\0';

	iter = pArtistList->begin();

	if (pArtistList->end() != iter)
	{
		artistFirstInterestingLetter		 = getFirstInterestingLetter((*iter)->GetName());
		artistFirstInterestingLetterPrevious = getFirstInterestingLetter((*iter)->GetName());

		m_artistPanels.back()->AddPanelLine(new CPLArtistsLetterHeading(artistFirstInterestingLetter));

		m_alphabetPanel.NoteArtistPanelNumber(artistFirstInterestingLetter, m_artistPanels.back()->GetNumber());
	}

	for (iter = pArtistList->begin(); pArtistList->end() != iter; iter++)
	{
		pArtist = (*iter);

		numTotalDiscs = pArtist->GetDiscList()->GetTotal();

		artistFirstInterestingLetter = getFirstInterestingLetter(pArtist->GetName());

		if (artistFirstInterestingLetterPrevious != artistFirstInterestingLetter)
		{
			if (bAddArtistsThisTime)
			{
				m_artistPanels.back()->AddPanelLine(new CPLTwoArtists(pArtistPrevious, artistDiscPanelNumberPrevious, NULL, 0));

				bAddArtistsThisTime = false;
			}

			m_artistPanels.back()->SetMsgRight(CCSFSection::BUTTON_JUMPARTISTPAGE, m_artistPanels.back()->GetNumber() + 1,
												"%c", artistFirstInterestingLetter);

			m_artistPanels.push_back(new CPanel('A', m_artistPanels.back()->GetNumber() + 1));
			m_artistPanels.back()->AddPanelLine(new CPLArtistsLetterHeading(artistFirstInterestingLetter));

			m_alphabetPanel.NoteArtistPanelNumber(artistFirstInterestingLetter, m_artistPanels.back()->GetNumber());

			m_artistPanels.back()->SetMsgLeft(CCSFSection::BUTTON_JUMPARTISTPAGE, m_artistPanels.back()->GetNumber() - 1,
												"%c", artistFirstInterestingLetterPrevious);
		}
		else if ((1 == m_artistPanels.back()->FreeLines()) && bAddArtistsThisTime)
		{
			m_artistPanels.back()->AddPanelLine(new CPLOneArtistAndContinuation(pArtistPrevious,
													artistDiscPanelNumberPrevious, m_artistPanels.back()->GetNumber() + 1));
			bAddArtistsThisTime = false;

			m_artistPanels.back()->SetMsgRight(CCSFSection::BUTTON_JUMPARTISTPAGE, m_artistPanels.back()->GetNumber() + 1,
												"...%c", artistFirstInterestingLetter);

			m_artistPanels.push_back(new CPanel('A', m_artistPanels.back()->GetNumber() + 1));
			m_artistPanels.back()->AddPanelLine(new CPLArtistsLetterContinuedHeading(artistFirstInterestingLetter,
														m_artistPanels.back()->GetNumber() - 1));

			m_artistPanels.back()->SetMsgLeft(CCSFSection::BUTTON_JUMPARTISTPAGE, m_artistPanels.back()->GetNumber() - 1,
												"%c...", artistFirstInterestingLetterPrevious);
		}

		artistDiscPanelNumber = doLayoutDiscs(pArtist, pArtist->GetDiscList(), pArtistPrevious, numTotalDiscs,
												m_artistPanels.back()->GetNumber());

		if (bAddArtistsThisTime)
		{
			bAddArtistsThisTime = false;

			m_artistPanels.back()->AddPanelLine(new CPLTwoArtists(pArtistPrevious, artistDiscPanelNumberPrevious,
																	pArtist,		artistDiscPanelNumber));
		}
		else
		{
			bAddArtistsThisTime = true;
		}

		pArtistPrevious = pArtist;
		artistDiscPanelNumberPrevious = artistDiscPanelNumber;
		artistFirstInterestingLetterPrevious = artistFirstInterestingLetter;
	}

	if (bAddArtistsThisTime)
	{
		m_artistPanels.back()->AddPanelLine(new CPLTwoArtists(pArtistPrevious, artistDiscPanelNumberPrevious, NULL, 0));
	}
}

// COMPACT--1	// Add a new panel if the current one does not have enough free
// COMPACT--1	// space for the entire artist, including a blank line before the heading.
// COMPACT--1	
// COMPACT--1	if ((4 + numTotal) > (2 * m_discPanels.back()->FreeLines()))
// COMPACT--1	{

// COMPACT--2	}
// COMPACT--2	else
// COMPACT--2	{
// COMPACT--2		// The current panel has room for all of the next artist following on from the previous artist.
// COMPACT--2		// Add a separator.
// COMPACT--2		m_discPanels.back()->AddPanelLine(new CPLBlank());
// COMPACT--2	}

// 1LM--1	bool bOneLineMode;
// 1LM--1
// 1LM--1	bOneLineMode = (numTotal <= m_discPanels.back()->FreeLines());
		
// 1LM--2	if (bOneLineMode)
// 1LM--2	{
// 1LM--2		m_discPanels.back()->AddPanelLine(new CPLOneDisc(pDiscOne));
// 1LM--2	}
// 1LM--2	else

// 1LM--3	bOneLineMode = ((numTotal - numDone) <= m_discPanels.back()->FreeLines());

// Returns number of first disc panel for the artist.
unsigned int CPanelList::doLayoutDiscs(const CArtist *pArtist, const CDiscList *pDiscList, const CArtist *pArtistPrevious,
										CDiscList::size_type numTotal, unsigned int artistArtistPanelNumber)
{
	// Add the main artist header, creating a new panel if desired / required.

	if (!m_discPanels.back()->IsEmpty())
	{
		// COMPACT--1

		// Always add a new panel for a new artist.
		// (I found the more compact layout confusing since you'd jump to an artist's page but they were not at the top.)
		m_discPanels.back()->SetMsgRight(CCSFSection::BUTTON_JUMPDISCPAGE, m_discPanels.back()->GetNumber() + 1,
												"%s", (const char *)(pArtist->GetShortName()));

		m_discPanels.push_back(new CPanel('D', m_discPanels.back()->GetNumber() + 1));

		if (NULL != pArtistPrevious)
		{
			m_discPanels.back()->SetMsgLeft(CCSFSection::BUTTON_JUMPDISCPAGE, m_discPanels.back()->GetNumber() - 1,
											"%s", (const char *)(pArtistPrevious->GetShortName()));
		}

		// COMPACT--2
	}

	unsigned int artistDiscPanelNumber = m_discPanels.back()->GetNumber();

	m_discPanels.back()->AddPanelLine(new CPLArtistHeading(pArtist, artistArtistPanelNumber));

	// Main header done, add those discs.
	
	CDiscList::size_type numDone = 0;

	const CDisc *pDiscOne;
	const CDisc *pDiscTwo;

	CDiscList::const_iterator iter = pDiscList->begin();

	// 1LM--1

	while ( numDone < numTotal )
	{
		pDiscOne = *iter++;
		numDone++;

		// ILM--2
		if ( ((numDone + 1) < numTotal) && (1 == m_discPanels.back()->FreeLines()) )
		{
			// We need a continuation message and new panel if the current panel has only one line left
			// and there are more than two discs left.

			m_discPanels.back()->AddPanelLine(new CPLOneDiscAndContinuation(pDiscOne, m_discPanels.back()->GetNumber() + 1));

			m_discPanels.back()->SetMsgRight(CCSFSection::BUTTON_JUMPDISCPAGE, m_discPanels.back()->GetNumber() + 1,
												"%s [%d]", (const char *)(pArtist->GetShortName()), (int)(numTotal - numDone));

			m_discPanels.push_back(new CPanel('D', m_discPanels.back()->GetNumber() + 1));

			m_discPanels.back()->SetMsgLeft(CCSFSection::BUTTON_JUMPDISCPAGE, m_discPanels.back()->GetNumber() - 1,
												"%s [%d]", (const char *)(pArtist->GetShortName()), (int)(numDone));

			m_discPanels.back()->AddPanelLine(new CPLArtistContinuedHeading(pArtist, m_discPanels.back()->GetNumber() - 1));

			// 1LM--3
		}
		else
		{
			if (numDone < numTotal)
			{
				pDiscTwo = *iter++;
				numDone ++;
			}
			else
			{
				pDiscTwo = NULL;
			}

			m_discPanels.back()->AddPanelLine(new CPLTwoDiscs(pDiscOne, pDiscTwo));
		}
	}

	return(artistDiscPanelNumber);
}

bool CPanelList::AddToRootCSFSection(CCSFSection *pRootCSFSection) const
{
	bool bResult = false;

	CCSFSection *pCSFSectionCDListDevice;

	linkCDDeviceToFirstArtistPanel(pRootCSFSection);

	if (setupCSFSectionEnvironment(pRootCSFSection))
	{
		const char *deviceTitle = "Device(\"CD List\")";

		// Remove any existing CD List device.

		while (NULL != (pCSFSectionCDListDevice = pRootCSFSection->FindSubSectionByName(deviceTitle)))
		{
			printf("WARNING: Existing CD List device replaced\n");
			pRootCSFSection->RemoveAndDeleteSubSectionByPointer(pCSFSectionCDListDevice);
		}

		// Add Our new CD List device.

		pCSFSectionCDListDevice = pRootCSFSection->AddSubSection();
		pCSFSectionCDListDevice->SetTitle(deviceTitle);

		addHardButtonsToCDListDevice(pCSFSectionCDListDevice);

		m_alphabetPanel.AddToDeviceCSFSection(pCSFSectionCDListDevice);

		m_creditsPanel.AddToDeviceCSFSection(pCSFSectionCDListDevice);

		list<CPanel *>::const_iterator iter;

		for (iter = m_artistPanels.begin(); m_artistPanels.end() != iter; iter++)
		{
			(*iter)->AddToDeviceCSFSection(pCSFSectionCDListDevice);
		}

		for (iter = m_discPanels.begin(); m_discPanels.end() != iter; iter++)
		{
			(*iter)->AddToDeviceCSFSection(pCSFSectionCDListDevice);
		}

		bResult = true;
	}

	return(bResult);
}

void CPanelList::linkCDDeviceToFirstArtistPanel(CCSFSection *pRootCSFSection) const
{
	CCSFSection *pCSFCDDevice = pRootCSFSection->FindSubSectionByName("Device(\"CD\")");

	if (NULL == pCSFCDDevice)
	{
		printf("WARNING: No \"CD\" device, no button linked to CD list\n");
	}
	else
	{
		unsigned int numberLinked = 0;

		CCSFSection *pCSFListButton;
		CCSFSection *pCSFAction;

		// Link all buttons labelled "List" which don't have an action at the moment.

		while (NULL != (pCSFListButton = pCSFCDDevice->FindButtonSubInAnySub("List", CCSFSection::BUTTONSEARCH_NOACTION)))
		{
			numberLinked++;

			// Add the new action.
			pCSFAction = pCSFListButton->AddSubSection();
			pCSFAction->SetTitle("Action()");
			pCSFAction->AddTitleOnlySubSection("JumpDevicePanel(\"CD List\", \"Alphabet\");");
		}

		if (0 != numberLinked)
		{
			printf("Linked inactive CD \"List\" button%s to CD List\n",
							(1 == numberLinked) ? "" : "s");
		}
		else
		{
			while (NULL != (pCSFListButton = pCSFCDDevice->FindButtonSubInAnySub("List", CCSFSection::BUTTONSEARCH_ACTIONWITHOUTALIAS)))
			{
				// Remove the existing action.
				pCSFAction = pCSFListButton->FindSubSectionByName("Action()");

				if (NULL != pCSFAction)
				{
					pCSFListButton->RemoveAndDeleteSubSectionByPointer(pCSFAction);
				}
			}

			// We can't do these two loops together because BUTTONSEARCH_ACTIONWITHOUTALIAS will keep finding
			// buttons with just a page-jump, which is what we're turning the buttons in to.
			while (NULL != (pCSFListButton = pCSFCDDevice->FindButtonSubInAnySub("List", CCSFSection::BUTTONSEARCH_NOACTION)))
			{
				numberLinked++;

				// Add the new action.
				pCSFAction = pCSFListButton->AddSubSection();
				pCSFAction->SetTitle("Action()");
				pCSFAction->AddTitleOnlySubSection("JumpDevicePanel(\"CD List\", \"Alphabet\");");
			}

			if (0 != numberLinked)
			{
				printf("WARNING: Replaced non-alias CD \"List\" button%s action%s\n",
							(1 == numberLinked) ? "'s" : "s'", (1 == numberLinked) ? "" : "s");
			}
			else
			{
				printf("WARNING: No inactive or non-alias CD \"List\" buttons, no link to CD List.\n");
			}
		}
	}
}

bool CPanelList::setupCSFSectionEnvironment(CCSFSection *pRootCSFSection) const
{
	bool bResult = true;

	CCSFSection *pCSFCDDevice = pRootCSFSection->FindSubSectionByName("Device(\"CD\")");

	if (NULL == pCSFCDDevice)
	{
		printf("ERROR: No \"CD\" device, cannot alias to number buttons\n");
		bResult = false;
	}
	else
	{
		CCSFButtonMemo *pButtonMemo;
		CCSFEnvironment *pCSFEnvironment = pRootCSFSection->GetEnvironment();

		// Disc button.

		pButtonMemo = findButtonAndCreateMemo(pCSFCDDevice, "Disc");

		if (NULL == pButtonMemo)
		{
			bResult = false;
		}
		else
		{
			pCSFEnvironment->SetBMDisc(pButtonMemo);
		}

		// Enter button.

		pButtonMemo = findButtonAndCreateMemo(pCSFCDDevice, "Enter");

		if (NULL == pButtonMemo)
		{
			bResult = false;
		}
		else
		{
			pCSFEnvironment->SetBMEnter(pButtonMemo);
		}

		// Number buttons.

		char textNumber[2];

		for (unsigned int i=0; i < 10; i++)
		{
			stringFormat(textNumber, sizeof(textNumber), "%1u", i);

			pButtonMemo = findButtonAndCreateMemo(pCSFCDDevice, textNumber);

			if (NULL == pButtonMemo)
			{
				bResult = false;
			}
			else
			{
				pCSFEnvironment->SetBMNumber(i, pButtonMemo);
			}
		}
	}

	return(bResult);
}

CCSFButtonMemo *CPanelList::findButtonAndCreateMemo(CCSFSection *pCSFSectionDevice, const char *name) const
{
	CCSFButtonMemo *result = NULL;

	if (NULL != name)
	{
		char *deviceName = NULL;
		char *panelName = NULL;
		char *buttonName = NULL;
		unsigned int buttonNumber = 0; // invalid button number, FWIW.

		if (! pCSFSectionDevice->ScanForDevice(&deviceName) )
		{
			printf("INTERNAL ERROR: Expected CSF device section\n");
		}
		else
		{
			CCSFSection *pCSFSectionButton;

			pCSFSectionButton = pCSFSectionDevice->FindButtonSubInAnySub(name, CCSFSection::BUTTONSEARCH_ACTIONWITHOUTALIAS);

			if (NULL == pCSFSectionButton)
			{
				printf("ERROR: %s%s%s device has no alias-free \"%s\" button\n",
							NULL == deviceName ? "<" : "\"",
							NULL == deviceName ? "unnamed" : deviceName,
							NULL == deviceName ? ">" : "\"",
							name);
			}
			else
			{
				// Get buttonName (should be equal to name) and buttonNumber.
				if (! pCSFSectionButton->ScanForButton(&buttonName, &buttonNumber) )
				{
					printf("INTERNAL ERROR: Found CSF button section which isn't a button\n");
				}
				else
				{
					CCSFSection *pCSFSectionPanel = pCSFSectionButton->GetParent();

					if (NULL == pCSFSectionPanel)
					{
						printf("INERNAL ERROR: Found CSF button section with no parent\n");
					}
					else if (! pCSFSectionPanel->ScanForPanel(&panelName) )
					{
						printf("INTERNAL ERROR: Found CSF button in section which isn't a panel\n");
						printf("                (New CSF format?)\n");
					}
					else
					{
						result = new CCSFButtonMemo(deviceName, panelName, buttonName, buttonNumber);

						delete [] panelName;
						panelName = NULL;
					}

					delete [] buttonName;
					buttonName = NULL;
				}
			}

			delete [] deviceName;
			deviceName = NULL;
		}
	}

	return(result);
}

void CPanelList::addHardButtonsToCDListDevice(CCSFSection *pCSFSectionCDListDevice) const
{
	CCSFSection *pHardButtonSection;

	pHardButtonSection = pCSFSectionCDListDevice->AddTitleOnlySubSection("Left(\"CD\")");
	pHardButtonSection->AddTitleOnlySubSection("JumpDevicePanel(\"CD\", \"Main\");");

	pHardButtonSection = pCSFSectionCDListDevice->AddTitleOnlySubSection("Right(\"A-Z\")");
	pHardButtonSection->AddTitleOnlySubSection("JumpDevicePanel(\"this\", \"Alphabet\");");

	pHardButtonSection = pCSFSectionCDListDevice->AddTitleOnlySubSection("Chan+()");
	pHardButtonSection->AddTitleOnlySubSection("ScrollUp();");

	pHardButtonSection = pCSFSectionCDListDevice->AddTitleOnlySubSection("Chan-()");
	pHardButtonSection->AddTitleOnlySubSection("ScrollDown();");

/*
	if (NULL != pCSFCDDevice)
	{
		pCSFSectionCDListDevice->MakeCopyOfSection(pCSFCDDevice, "Vol-()");
		pCSFSectionCDListDevice->MakeCopyOfSection(pCSFCDDevice, "Vol+()");
		pCSFSectionCDListDevice->MakeCopyOfSection(pCSFCDDevice, "Mute()");
	}
*/
}

/////////////////////////////////////////////////////////////////////////
