
#ifndef __PANELLIST_H_
#define __PANELLIST_H_

#include "CSFSection.h"

/////////////////////////////////////////////////////////////////////////

class CPanelLine
{
public:
	CPanelLine(void)
	{
	}

	virtual ~CPanelLine(void)
	{
	}

	virtual void AddToPanelCSFSection(CCSFSection *pPanelCSFSection,  unsigned int ypos) const = 0;
};

/////////////////////////////////////////////////////////////////////////

class CPanel
{
public:
	CPanel(char prefixChar, unsigned int panelNumber);
	virtual ~CPanel(void);

	char GetPrefixChar(void);
	unsigned int GetNumber(void);

	void SetMsgLeft(CCSFSection::ButtonType linkType, unsigned int linkNumber, const char *msg, ...);
	void SetMsgRight(CCSFSection::ButtonType linkType, unsigned int linkNumber, const char *msg, ...);

	unsigned int FreeLines(void) const;
	bool IsEmpty(void) const;

	void AddPanelLine(CPanelLine *pPanelLine);

	enum { MAX_LINES = 6 };

	void AddToDeviceCSFSection(CCSFSection *pDeviceCSFSection) const;

protected:
	char m_prefixChar;
	unsigned int m_panelNumber;

	char *					m_msgLeft;
	CCSFSection::ButtonType	m_msgLeftLinkType;
	unsigned int			m_msgLeftLinkNumber;

	char *					m_msgRight;
	CCSFSection::ButtonType	m_msgRightLinkType;
	unsigned int			m_msgRightLinkNumber;
	
	int m_numFreeLines;

	list<CPanelLine *> m_panelLines;
};

/////////////////////////////////////////////////////////////////////////

class CAlphaPanelLetter
{
public:
	CAlphaPanelLetter(char letter, unsigned int artistPanelNumber);
	virtual ~CAlphaPanelLetter(void);

	char GetArtistPanelLetter(void);
	unsigned int GetArtistPanelNumber(void);

protected:
	char m_letter;
	unsigned int m_artistPanelNumber;
};

class CAlphabetPanel
{
public:
	CAlphabetPanel(void);
	virtual ~CAlphabetPanel(void);

	void AddToDeviceCSFSection(CCSFSection *pDeviceCSFSection) const;

	void NoteArtistPanelNumber(const char letterToFile, unsigned int artistPanelNumber);

protected:

	enum { LETTER_ARRAY_SIZE = 27 };
	enum { LETTER_ARRAY_ALPHA_OFFSET = 1 };

	CAlphaPanelLetter *m_letterArray[LETTER_ARRAY_SIZE];
};

/////////////////////////////////////////////////////////////////////////

class CCreditsPanel
{
public:
	CCreditsPanel(void);
	virtual ~CCreditsPanel(void);

	void AddToDeviceCSFSection(CCSFSection *pDeviceCSFSection) const;
};

/////////////////////////////////////////////////////////////////////////

class CPanelList
{
public:
	CPanelList(void);
	virtual ~CPanelList(void);

	void DoLayout(const CArtistList *pArtistList);

	bool AddToRootCSFSection(CCSFSection *pRootCSFSection) const;

protected:
	void doLayoutArtists(const CArtistList *pArtistList);

	// Returns number of first disc panel for the artist.
	unsigned int doLayoutDiscs(const CArtist *pArtist, const CDiscList *pDiscList, const CArtist *pArtistPrevious,
						CDiscList::size_type numTotal, unsigned int artistArtistPanelNumber);

	void linkCDDeviceToFirstArtistPanel(CCSFSection *pRootCSFSection) const;
	bool setupCSFSectionEnvironment(CCSFSection *pRootCSFSection) const;
	CCSFButtonMemo *findButtonAndCreateMemo(CCSFSection *pCSFSectionDevice, const char *name) const;
	void addHardButtonsToCDListDevice(CCSFSection *pCSFSectionCDListDevice) const;

	CAlphabetPanel m_alphabetPanel;
	CCreditsPanel m_creditsPanel;

	list<CPanel *> m_artistPanels;
	list<CPanel *> m_discPanels;
};

/////////////////////////////////////////////////////////////////////////

#endif

