
#include "stdafx.h"

// strncpy doesn't always null term the destination... sigh.
// This returns the index of the null terminator.
int stringCopy(char *dest, const char *source, size_t destSize)
{
	int idx = 0;

	if (destSize > 0)
	{
		while (('\0' != source[idx]) && (destSize > 1))
		{
			dest[idx] = source[idx];
			idx++;
			destSize--;
		}

		dest[idx] = '\0';
	}

	return(idx);
}

// _snprintf doesn't always null term the destination... sigh
// Returns the index of the null terminator.
int stringFormat(char *buffer, size_t bufferSize, const char *format, ...)
{
	va_list args;
	va_start(args, format);
	int result = vStringFormat(buffer, bufferSize, format, args);
	va_end(args);

	return(result);
}

// _vsnprintf doesn't always null term the destination... sigh
// Returns the index of the null terminator.
int vStringFormat(char *buffer, size_t bufferSize, const char *format, va_list args)
{
	int result = 0;

	if (bufferSize > 0)
	{
		result = _vsnprintf(buffer, bufferSize, format, args);

		if (0 > result)
		{
			result = bufferSize - 1;
			buffer[result] = '\0';
		}
	}

	return(result);
}

char *TrimWhitespace(char *buffer)
{
	while(isspace(*buffer))
	{
		buffer++;
	}

	char *bufEnd = buffer + strlen(buffer);

	while( (bufEnd > buffer) && (isspace(*(--bufEnd))) )
	{
		*bufEnd = '\0';
	}

	return(buffer);
}

// pp -- Pointer to buffer to tokenize. Will be advanced to the next position string.
// delimiters -- Characters which delimit the string.
// skipEmpty -- Set true to skip any empty strings. (e.g. someone typed "blah,,blah")
// buf -- Buffer to extract in to. Set to NULL not to extract.
// bufSize -- Size of buf if it is non-NULL. Will truncate string if buf too small.
// Leading and trailing spaces are always stripped.
// Returns false iff nothing could be extracted (i.e. *pp was effectively empty)
// pp and the contents of buf are only valid after calling if true is returns.
// Lame function should allocate us a suitably sized buffer for each string... :-)
// But hey, this isn't a public OS function and you can just give a buffer the size of the input...
bool TokenizeEx(const char **pp, const char *delimiters, char *buf, size_t bufSize, bool skipEmpty)
{
	bool result = false;

	if (NULL != *pp)
	{
		// Remove leading spaces.
		// Also, skip leading delimiters if we're to skip empties.
		// (Important that we skip the spaces here so that they're not included if it's the first string.)
		// (Note we must explicitly check for the terminator before checking the delimiters else we ask if
		// the terminator is in the delimiters string.)
		while ( isspace(**pp) || (('\0' != **pp) && skipEmpty && (NULL != strchr(delimiters, **pp))) )
		{
			(*pp)++;
		}

		if ('\0' != **pp)
		{
			char *bufOrig = buf;

			result = true; // We got somethin'!

			// There is some string left, extract what's next.
			while ( ('\0' != **pp) && (NULL == strchr(delimiters, **pp)) )
			{
				if ((NULL != buf) && (bufSize > 1)) // Keep 1 character for the terminator.
				{
					*buf = **pp;
					bufSize--;
					buf++;
				}

				(*pp)++; // Must increment *pp here as copying to buf is optional and may also be trunctated.
			}

			// Handles the case where a zero-length buffer is passed in.
			if ((NULL != buf) && (bufSize > 0))
			{
				*buf = '\0'; // NULL-terminate it.

				// Remove trailing spaces.
				while (buf > bufOrig)
				{
					if (isspace(*(--buf)))
					{
						*buf = '\0';
					}
					else
					{
						break;
					}
				}
			}

			// LD 11/Mar/2000
			// Fixed but which skipped empty columns after the first column. We only skip the immediate delimeter,
			// even if there's nothing but spaces between it and the next.

			// Move over the delimiter (unless we're at the end of the string).
			if (('\0' != **pp) && (NULL != strchr(delimiters, **pp)))
			{
				(*pp)++;
			}

			// Skip spaces so that when we extract the last string we return *pp as an empty string.
			// (Useful for when buf==NULL)
			while (('\0' != **pp) && isspace(**pp))
			{
				(*pp)++;
			}
		}
	}

	return(result);
}

// returns NULL on failure
// delete[] the result when you are finished with it.
char *getFileVersion(void)
{
	char *result = NULL;
	TCHAR modName[_MAX_PATH + 1];
	DWORD verSize;
	DWORD whatTheHellIsThePointOfThisVariableWhichJustGetsSetToZero;
	// Since the documentation to GetFileVersionInfoSize() says the second argument is always set to zero,
	// they must never actually do anything useful with it because I am perfectly in my rights to use
	// and rely on GetFileVersionInfoSize() when I want to set a DWORD variable to zero.
	// Bad documentation or bad code? You decide... :)

	if ((GetModuleFileName(NULL, modName, _MAX_PATH))
	&&  (0 != (verSize = GetFileVersionInfoSize(modName, &whatTheHellIsThePointOfThisVariableWhichJustGetsSetToZero))))
	{
		BYTE *verData = new BYTE[verSize];

		if (GetFileVersionInfo(modName, NULL, verSize, verData))
		{
			LPVOID pBuffer;
			UINT bufLen;

			if(VerQueryValue(verData, "\\VarFileInfo\\Translation", &pBuffer, &bufLen))
			{
				// Find out language of version resource
				BYTE* lang = (BYTE*)pBuffer;

				TCHAR resblk[BUFFERSIZE];
				stringFormat(resblk, sizeof(resblk), "\\StringFileInfo\\%02x%02x%02x%02x\\FileVersion", lang[1], lang[0], lang[3], lang[2]);

				if (VerQueryValue(verData, resblk, &pBuffer, &bufLen))
				{
					size_t resLen = 1 + 4 + strlen((const char *)pBuffer);
					result = new char[resLen];

					if (NULL != result)
					{
						int n1, n2, n3, n4;
						switch(sscanf((TCHAR *)pBuffer, "%d,%d,%d,%d", &n1, &n2, &n3, &n4))
						{
						case 1:		n2 = 0;
						case 2:		n3 = 0;
						case 3:		n4 = 0;
						case 4:		stringFormat(result, resLen, "%d.%d.%d.%d", n1, n2, n3, n4);
									break;
						default:	delete [] result;
									result = NULL;
									break;
						}
					}
				}
			}

		}

		delete [] verData;
	}

	return(result);
}

// If this returns '\0' there was no alpha-numeric letter.
char getFirstInterestingLetter(const char *string)
{
	char cResult = '\0';

	if (NULL != string)
	{
		while ('\0' != *string)
		{
			if (isalnum(*string))
			{
				cResult = (*string);

				if (islower(cResult))
				{
					cResult = toupper(cResult);
				}
				else if (isdigit(cResult))
				{
					cResult = '#';
				}

				break;
			}

			string++;
		}
	}

	return(cResult);
}
