
#include "stdafx.h"

#include "Discography.h"
#include "CDListFile.h"
#include "SimpleTextFile.h"
#include "CSFSection.h"
#include "PanelList.h"
#include "main.h"

int main(int argc, char *argv[])
{
	if (4 != argc)
	{
		char *fileVersion = getFileVersion();

		printf("\n");
		printf("Nudelist CD Megachanger list creator for Pronto\n");
		printf("(C)2000 Leo Davidson. Freeware. C++ source available.\n");

		if (NULL != fileVersion)
		{
			printf("Version %s\n", fileVersion);
		}

		printf("\n");
		printf("usage:\n");
		printf("nudelist <cd list> <base input csf> <output csf>\n");
		printf("\n");
		printf("Feel free to mail me: leo@ox.compsoc.net\n");
		printf("Updates should appear at http://www.ox.compsoc.net/~leo\n");
		printf("and will also be posted to the great http://www.remotecentral.com\n");

		if (NULL != fileVersion)
		{
			delete [] fileVersion;
			fileVersion = NULL;
		}
	}
	else
	{
		FILE *pfInputList = NULL;
		FILE *pfInputBaseCSF = NULL;
		FILE *pfOutputCSF = NULL;

		if (NULL == (pfInputList = fopen(argv[1], "r")))
		{
			printf("ERROR: Cannot read from %s\n", argv[1]);
		}
		else if (NULL == (pfInputBaseCSF = fopen(argv[2], "r")))
		{
			printf("ERROR: Cannot read from %s\n", argv[2]);
		}
		else if (NULL == (pfOutputCSF = fopen(argv[3], "w")))
		{
			printf("ERROR: Cannot create new file %s\n", argv[3]);
		}
		else
		{
			CArtistList *pArtistList = new CArtistList();
			CCSFSection *pRootCSFSection = new CCSFSection(NULL); // The root.

			if (readCDList(pfInputList, pArtistList))
			{
				printf("CD list read\n");

				if (readBaseCSF(pfInputBaseCSF, pRootCSFSection))
				{
					printf("Base CSF read\n");

					CPanelList *pPanelList = new CPanelList();

					pPanelList->DoLayout(pArtistList);
					printf("Layout done\n");

					if (pPanelList->AddToRootCSFSection(pRootCSFSection))
					{
						printf("Merge done\n");

						if (pRootCSFSection->Write(pfOutputCSF))
						{
							printf("CSF output\n");
							printf("Completed.\n");
						}
					}

					delete pPanelList;
				}
			}

			delete pRootCSFSection;
			delete pArtistList;
		}

		if (NULL != pfInputList)
		{
			fclose(pfInputList);
			pfInputList = NULL;
		}

		if (NULL != pfInputBaseCSF)
		{
			fclose(pfInputBaseCSF);
			pfInputBaseCSF = NULL;
		}

		if (NULL != pfOutputCSF)
		{
			fclose(pfOutputCSF);
			pfOutputCSF = NULL;
		}
	}

	return(0);
}

bool readCDList(FILE *pfInputList, CArtistList *pArtistList)
{
	bool bResult = false;

	CCDListFile *pListFile = new CCDListFile;

	if (pListFile->Read(pfInputList)			\
	&&	pListFile->Parse()						\
	&&	pListFile->MakeArtistList(pArtistList)	)
	{
		bResult = true;
	}

	delete pListFile;
	pListFile = NULL;

	return(bResult);
}

bool readBaseCSF(FILE *pfInputBaseCSF, CCSFSection *pBaseInputCSFSection)
{
	bool bResult = false;

	CSimpleTextFile *pTextFile = new CSimpleTextFile;

	if (pTextFile->Read(pfInputBaseCSF))
	{
		CSimpleTextFile::const_lineIterator lineIter = pTextFile->lineBegin();
		CSimpleTextFile::const_lineIterator lineIterEnd = pTextFile->lineEnd();

		if (pBaseInputCSFSection->ReadNextSection(lineIter, lineIterEnd))
		{
			bResult = true;
		}
	}

	delete pTextFile;
	pTextFile = NULL;

	return(bResult);
}
